//
//  CHClipArtController.h
//  AddressBookStudio
//
//  Created by Robert McCullough on Tue Sep 10 2002.
//  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHFoundation/CHFoundation.h>
#import "CHNibLoadingObject.h"

@class CHClipArtMatrix, CHClipArtImageLoader;
@class CHSearchField;
@class CHQueue;

enum {
	kClipArtControllerType = 0,
	kImageMasksControllerType,
	kIPhotoControllerType
};

@protocol ClipArtImagesProtocol
- (void)setImageArray:(NSArray*)newImageArray indexArray:(NSArray*)newImageIndexArray minCellWithImage:(int)minCellWithImage maxCellWithImage:(int)maxCellWithImage imageLoader:(CHClipArtImageLoader*)imageLoader;
//- (NSImage *)getImage:(NSString *)imagePath;
@end

@interface CHClipArtController : CHNibLoadingObject
{
	IBOutlet id _mainView;
	IBOutlet NSTableView *_categoryTable;
	IBOutlet CHClipArtMatrix *_clipArtMatrix;
	IBOutlet NSSlider *_zoomSlider;
	IBOutlet NSView *_zoomSliderView;
	
	IBOutlet NSView *_searchGroup;
	IBOutlet CHSearchField *_searchField;
	IBOutlet NSTextField *_searchCount;
	
	IBOutlet NSPopUpButton *_searchOptionsButton;
	
	//IBOutlet NSPopUpButton *_shapesPopupButton;
	IBOutlet NSButton *_objectMaskButton;
	//IBOutlet NSButton *_backgroundMaskSwitch;
	
	IBOutlet NSProgressIndicator *_progressBar;
	IBOutlet NSView *_progressBarGroup;
	
	//NSMutableArray *_topLevelNibObjects;
	
	NSMutableArray *_clipArtArrays;
	NSMutableArray *_clipArtFolderPathArray;	// contains a full path to the clip art folder containing the clip art array in _clipArtArrays
	NSMutableArray *_titleKeywordIndexArrays;
	NSMutableArray *_categoryIndexArrays;
	NSMutableArray *_categoryArray;
	NSArray *_photoKeyArray;
	//7/15NSMutableArray *_photoCategoryArray;

	NSMutableArray *_selectedIndexArray;
	NSMutableArray *_imageArray;
	NSMutableArray *_imageIndexArray;
	NSMutableArray *_imageLoaderArray;

	BOOL _matchAllWords;
	int _minCellWithImage;
	int _maxCellWithImage;
	
	int _controllerType;

	CHQueue *_imageLoaderQueue;

	BOOL _ignoreMatrixResizeNotification;

	id _editController;
	
	//BOOL _shouldEnableObjectMask;
	//BOOL _shouldEnableBackgroundMask;
	
	BOOL _finishedLoading;
	BOOL _clipArtCategoryIsSelected;
}

- (id)initWithType:(int)controllerType;

- (id)mainView;
- (void)setEditController:(id)controller;
//7/15- (void)setDrawerController:(ASDrawerController *)controller;

- (float)zoomFactor;
- (NSRect)topPageRect;
- (void)setSizeImageToTarget:(BOOL)sizeImageToTarget;

- (void)drawerResized;
- (IBAction)setZoomFactor:(id)sender;
- (void)matrixBoundsChanged:(NSNotification *)aNotification;
- (IBAction)searchOptionsAction:(id)sender;

/*- (void)graphicSelectionChanged;
- (IBAction)maskAction:(id)sender;*/
- (void)selectionChangedToRow:(NSInteger)row column:(NSInteger)column;
- (void)processDoubleClick;

- (void)processSearchStr:(NSNotification *)notification;
- (NSDictionary *)clipArtDictionaryAtIndex:(NSInteger)index folderIndex:(NSInteger *)folderIndex;
- (NSArray *)recordsMatchingWord:(NSString *)word usingIndex:(NSArray *)indexArray;
- (void)addRecordIndexArray:(NSArray *)array1 toArray:(NSMutableArray *)array2;
- (void)unionRecordIndexArray:(NSArray *)array1 toArray:(NSMutableArray *)array2;
- (void)loadSelectedImages;
- (void)setImageArray:(NSArray*)newImageArray indexArray:(NSArray*)newImageIndexArray minCellWithImage:(int)minCellWithImage maxCellWithImage:(int)maxCellWithImage imageLoader:(CHClipArtImageLoader*)imageLoader;
- (void)adjustNumCellsInMatrix;
- (void)populateMatrixCells;
//7/15- (int)selectedPhotoCategoryIndex;

/*- (int)numberOfRowsInTableView:(NSTableView *)aTableView;
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;*/

- (void)loadClipArt;
- (void)cleanupAfterLoading;

@end
