//
//  CHMonthView.h
//  CHAppKit
//
//  Created by Nick Zitzmann on 11/28/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHFoundation/CHFoundation.h>

#if MAC_OS_X_VERSION_10_4 <= MAC_OS_X_VERSION_MAX_ALLOWED
typedef enum
{
	CHMVDLTopLeft = 0,
	CHMVDLTopRight,
	CHMVDLBottomLeft,
	CHMVDLBottomRight
} CHMonthViewDayLocation;

typedef enum
{
	CHMVSingleLineGridStyle = 0
} CHMonthViewGridStyle;

typedef enum
{
	CHMVNoTimeFormat = 0,
	CHMVStartTimeFormat,
	CHMVStartAndEndTimeFormat
} CHMonthViewEventTimeFormat;

@interface CHMonthView : NSView
{
	NSCalendar *_calendar;
	NSUInteger _numberOfRowsInCalendar;
	NSDate *_startDate;
	NSDate *_endDate;
	BOOL _inSMVMode;	// SMV = "Super Month View"
	NSInteger _activeMonth;
	NSIndexSet *_businessDays;
	NSDate *_selectedDate;
	IBOutlet id _nonretainedDataSource;
	IBOutlet id _nonretainedDelegate;
	
	// The events:
	NSMutableDictionary *_allDayEventCellsKeyedByDate;
	NSMutableDictionary *_timedEventCellsKeyedByDate;
	NSMutableDictionary *_eventCellInfoDictsKeyedByDay;
	NSMutableSet *_datesThatRequireEllipsesInTheirRects;
	BOOL _needsReload;
	NSDictionary *_eventCellInfoDictAtMouseDown;
	NSMutableArray *_draggingEventCellsLeft;
	BOOL _actuallyDraggedCellsLeft;
	NSMutableArray *_draggingEventCellsRight;
	BOOL _actuallyDraggedCellsRight;
	NSCalendarDate *_draggingEventCellsBothOriginalStartDate;
	BOOL _draggingEventCellsBothDuplicate;
	NSMutableArray *_incomingDragCells;
	NSDragOperation _incomingDragOp;
	BOOL _incomingDragIsNew;
	
	// Drawing options:
	CHMonthViewDayLocation _dayLocation;
	CHMonthViewGridStyle _gridStyle;
	float _gridHorizontalLineThickness;
	float _gridVerticalLineThickness;
	BOOL _drawOnlyBusinessDays;
	BOOL _drawDayInYearNumbers;
	NSFont *_drawDayFont;
	NSColor *_drawDayInYearColor;
	BOOL _drawWeekInYearNumbers;
	NSFont *_drawWeekInYearFont;
	NSColor *_drawWeekInYearColor;
	NSFont *_drawMonthInSMVFont;
	NSColor *_drawMonthInSMVColor;
	BOOL _drawTitleRegion;
	CGFloat _drawTitleRegionHeight;
	NSFont *_drawTitleFont;
	NSColor *_drawTitleColor;
	BOOL _drawWeekdayRegion;
	CGFloat _drawWeekdayRegionHeight;
	NSFont *_drawWeekdayFont;
	NSColor *_drawWeekdayColor;
	NSFont *_eventCellFont;
	
	CHMonthViewEventTimeFormat _eventTimeFormat;
	NSImage *_backgroundImage;
	BOOL _drawBackground;
	NSColor *_backgroundColor;
}
- (NSCalendar *)calendar;
- (void)setCalendar:(NSCalendar *)newCalendar;
- (NSDateComponents *)eraYearMonthWeek;
- (void)setEra:(NSInteger)era year:(NSInteger)year month:(NSInteger)month;	// use this to display only one month
- (void)setEra:(NSInteger)era year:(NSInteger)year weekRange:(NSRange)weekRange;	// use this to display a series of weeks, which may span more than one month ("Super Month View")
- (NSIndexSet *)businessDays;
- (void)setBusinessDays:(NSIndexSet *)newBusinessDays;
- (NSDate *)selectedDate;
- (void)setSelectedDate:(NSDate *)newSelected;
- (id)delegate;
- (void)setDelegate:(id)newDelegate;
- (id)dataSource;
- (void)setDataSource:(id)newDataSource;

// Rows & columns
- (NSUInteger)numberOfColumns;
- (NSUInteger)numberOfRows;
- (NSCalendarDate *)dateAtPoint:(NSPoint)point;
- (NSCalendarDate *)dateAtRow:(NSUInteger)row column:(NSUInteger)column;

// Events:
- (void)loadAdditionOfEvents:(NSArray *)events;
- (void)loadDeletionOfEvents:(NSArray *)events;
- (void)loadModificationOfEvents:(NSArray *)events;
- (NSUInteger)numberOfEvents;
- (void)reloadData;
- (void)selectEvents:(NSArray *)events onStartDates:(NSArray *)startDates byExtendingSelection:(BOOL)extendSelection;
- (NSArray *)selectedEvents;
- (NSArray *)startDatesOfSelectedEvents;

// Drawing customization:
- (CHMonthViewDayLocation)dayLocation;
- (void)setDayLocation:(CHMonthViewDayLocation)newLocation;
- (CHMonthViewGridStyle)gridStyle;
- (void)setGridStyle:(CHMonthViewGridStyle)gridStyle;
- (float)gridHorizontalLineThickness;
- (void)setGridHorizontalLineThickness:(float)newThickness;
- (float)gridVerticalLineThickness;
- (void)setGridVerticalLineThickness:(float)newThickness;
- (BOOL)drawsOnlyBusinessDays;
- (void)setDrawsOnlyBusinessDays:(BOOL)drawOnly;
- (BOOL)drawsDayInYearNumbers;
- (void)setDrawsDayInYearNumbers:(BOOL)draw;
- (NSFont *)dayNumbersFont;
- (void)setDayNumbersFont:(NSFont *)font;
- (NSColor *)dayInYearNumbersColor;
- (void)setDayInYearNumbersColor:(NSColor *)color;
- (NSFont *)monthInSuperMonthViewFont;
- (void)setMonthInSuperMonthViewFont:(NSFont *)font;
- (NSColor *)monthInSuperMonthViewColor;
- (void)setMonthInSuperMonthViewColor:(NSColor *)color;
- (BOOL)drawsTitleRegion;
- (void)setDrawsTitleRegion:(BOOL)draw;
- (CGFloat)titleRegionHeight;
- (NSFont *)titleRegionFont;
- (void)setTitleRegionFont:(NSFont *)font;
- (NSColor *)titleRegionColor;
- (void)setTitleRegionColor:(NSColor *)color;
- (BOOL)drawsWeekdayRegion;
- (void)setDrawsWeekdayRegion:(BOOL)draw;
- (CGFloat)weekdayRegionHeight;
- (NSFont *)weekdayRegionFont;
- (void)setWeekdayRegionFont:(NSFont *)font;
- (NSColor *)weekdayRegionColor;
- (void)setWeekdayRegionColor:(NSColor *)color;
- (BOOL)drawsWeekInYearNumbers;
- (void)setDrawsWeekInYearNumbers:(BOOL)draw;
- (NSFont *)weekInYearFont;
- (void)setWeekInYearFont:(NSFont *)font;
- (NSColor *)weekInYearColor;
- (void)setWeekInYearColor:(NSColor *)color;
- (NSImage *)backgroundImage;
- (void)setBackgroundImage:(NSImage *)background;
- (CHMonthViewEventTimeFormat)eventTimeFormat;
- (void)setEventTimeFormat:(CHMonthViewEventTimeFormat)timeFormat;
- (NSFont *)eventCellFont;
- (void)setEventCellFont:(NSFont *)eventCellFont;
- (BOOL)drawsBackground;
- (void)setDrawsBackground:(BOOL)draw;
- (NSColor *)backgroundColor;
- (void)setBackgroundColor:(NSColor *)bgColor;

@end

@interface NSObject (CHMonthViewDataSourceInformal)
//- (BOOL)monthView:(CHMonthView *)monthView acceptDrop:(id <NSDraggingInfo>)info newStartDate:(NSCalendarDate *)newStartDate;
- (BOOL)monthView:(CHMonthView *)monthView finalizeDropOfEvent:(id <CHCalendarEvent>)event atNewStartDate:(NSCalendarDate *)newStartDate isNew:(BOOL)isNew;
- (id <CHCalendarEvent>)monthView:(CHMonthView *)monthView duplicateCalendarEvent:(id <CHCalendarEvent>)calendarEvent;
- (NSArray *)monthView:(CHMonthView *)monthView eventsInRangeFromStartDate:(NSDate *)startDate toEndDate:(NSDate *)endDate;
//- (NSDragOperation)monthView:(CHMonthView *)monthView validateDrop:(id <NSDraggingInfo>)info;
- (id <CHCalendarEvent>)monthView:(CHMonthView *)monthView preValidateDrop:(id <NSDraggingInfo>)info atStartDate:(NSCalendarDate *)startDate dragOperationToUse:(NSDragOperation *)dragOperation eventIsNew:(BOOL *)isNew;
- (void)monthView:(CHMonthView *)monthView writeEvent:(id <CHCalendarEvent>)event onDate:(NSCalendarDate *)startDate toPasteboard:(NSPasteboard *)pb;
@end

@interface NSObject (CHMonthViewDelegateInformal)
- (BOOL)monthViewCanCreateEvent:(CHMonthView *)monthView;
- (id <CHCalendarEvent>)monthView:(CHMonthView *)monthView createEventWithStartDate:(NSCalendarDate *)startDate endDate:(NSCalendarDate *)endDate summary:(NSString *)summary;
- (void)monthView:(CHMonthView *)monthView eventsWereSelected:(NSArray *)events clickCount:(NSInteger)clickCount;
- (void)monthView:(CHMonthView *)monthView modifyEvent:(id <CHCalendarEvent>)calendarEvent originalStartDate:(NSCalendarDate *)originalStartDate newStartDate:(NSCalendarDate *)newStartDate endDate:(NSCalendarDate *)newEndDate summary:(NSString *)newSummary;
- (void)monthView:(CHMonthView *)monthView selectedDateDidChange:(NSDate *)newSelectedDate;
- (BOOL)monthView:(CHMonthView *)monthView shouldAllowDragOfEvent:(id <CHCalendarEvent>)event;
- (NSString *)monthView:(CHMonthView *)monthView toolTipForEvent:(id <CHCalendarEvent>)event;
@end
#endif
