//
//  CHMultidayView.h
//  CHAppKit
//
//  Created by Nick Zitzmann on 9/14/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHFoundation/CHFoundation.h>

#if MAC_OS_X_VERSION_10_4 <= MAC_OS_X_VERSION_MAX_ALLOWED
@class CHMultidayAllDayView, CHMultidayTimeView;

@interface CHMultidayView : NSView
{
	NSCalendar *_calendar;
	NSCalendarDate *_startDate;
	NSUInteger _numberOfDays;
	int _dayStartHour;
	int _dayEndHour;
	unsigned int _dragIncrementInMinutes;
	float _zoomFactor;
	NSIndexSet *_businessDays;
	NSDate *_selectedDate;
	IBOutlet id _nonretainedDataSource;
	IBOutlet id _nonretainedDelegate;
	
	NSSplitView *_splitView;
	CHMultidayAllDayView *_allDayView;
	NSScrollView *_timeScrollView;
	CHMultidayTimeView *_timeView;
	
	BOOL _needsReload;
}
- (id)initWithFrame:(NSRect)frame calendar:(NSCalendar *)calendar startDate:(NSDate *)date numberOfDays:(unsigned int)numberOfDays dayStartHour:(int)dayStartHour dayEndHour:(int)dayEndHour zoomFactor:(float)zoomFactor businessDays:(NSIndexSet *)businessDays;

- (NSCalendar *)calendar;
- (void)setCalendar:(NSCalendar *)calendar;
- (NSCalendarDate *)startDate;
- (void)setStartDate:(NSDate *)startDate;
- (NSUInteger)numberOfColumns;
- (NSUInteger)numberOfDays;
- (void)setNumberOfDays:(NSUInteger)numberOfDays;
- (int)dayStartHour;
- (int)dayEndHour;
- (void)setDayStartHour:(int)dayStartHour endHour:(int)dayEndHour;
- (unsigned int)dragIncrementInMinutes;
- (void)setDragIncrementInMinutes:(unsigned int)minutes;
- (float)zoomFactor;
- (void)setZoomFactor:(float)factor;
- (NSIndexSet *)businessDays;
- (void)setBusinessDays:(NSIndexSet *)businessDays;
- (NSDate *)selectedDate;
- (void)setSelectedDate:(NSDate *)selectedDate;
- (NSRect)scrollRectOfTimeView;
- (void)setScrollRectOfTimeView:(NSRect)normalizedVisibleRect;
- (id)dataSource;
- (void)setDataSource:(id)dataSource;
- (id)delegate;
- (void)setDelegate:(id)delegate;
- (NSTableHeaderCell *)headerCell;
- (void)setHeaderCell:(NSTableHeaderCell *)headerCell;
- (NSView *)cornerView;
- (void)setCornerView:(NSView *)cornerView;
- (BOOL)showsCurrentTime;
- (void)setShowsCurrentTime:(BOOL)showsCurrentTime;

- (void)xLocation:(CGFloat *)location width:(CGFloat *)width ofColumn:(NSInteger)columnIndex;
- (NSCalendarDate *)dateForColumn:(NSInteger)columnIndex;
- (void)loadAdditionOfEvents:(NSArray *)newEvents;
- (void)loadDeletionOfEvents:(NSArray *)deletedEvents;
- (void)loadModificationOfEvents:(NSArray *)existingEvents;
- (NSUInteger)numberOfEvents;
- (void)reloadData;
- (void)scrollHourToVisible:(unsigned int)hourIndex;
- (NSArray *)selectedEvents;
- (NSArray *)startDatesOfSelectedEvents;
- (void)selectEvents:(NSArray *)events onStartDates:(NSArray *)startDates byExtendingSelection:(BOOL)extendSelection;
@end

@interface NSObject (CHMultidayViewDataSourceInformal)
//- (BOOL)multidayView:(CHMultidayView *)multidayView acceptDrop:(id <NSDraggingInfo>)info newStartDate:(NSCalendarDate *)newStartDate allDay:(BOOL)allDay;
- (BOOL)multidayView:(CHMultidayView *)multidayView finalizeDropOfEvent:(id <CHCalendarEvent>)event atNewStartDate:(NSCalendarDate *)newStartDate isNew:(BOOL)isNew;
- (id <CHCalendarEvent>)multidayView:(CHMultidayView *)multidayView duplicateCalendarEvent:(id <CHCalendarEvent>)calendarEvent;
- (NSArray *)multidayView:(CHMultidayView *)multidayView eventsInRangeFromStartDate:(NSDate *)startDate toEndDate:(NSDate *)endDate;	// must be implemented
//- (NSDragOperation)multidayView:(CHMultidayView *)multidayView validateDrop:(id <NSDraggingInfo>)info;
- (id <CHCalendarEvent>)multidayView:(CHMultidayView *)multidayView preValidateDrop:(id <NSDraggingInfo>)info atStartDate:(NSCalendarDate *)startDate allDay:(BOOL)allDay dragOperationToUse:(NSDragOperation *)dragOperation eventIsNew:(BOOL *)isNew;
- (void)multidayView:(CHMultidayView *)multidayView writeEvent:(id <CHCalendarEvent>)calendarEvent onDate:(NSCalendarDate *)startDate toPasteboard:(NSPasteboard *)pb;
@end

@interface NSObject (CHMultidayViewDelegateInformal)
- (BOOL)multidayViewCanCreateEvent:(CHMultidayView *)multidayView;
- (id <CHCalendarEvent>)multidayView:(CHMultidayView *)multidayView createEventWithStartDate:(NSCalendarDate *)startDate endDate:(NSCalendarDate *)endDate summary:(NSString *)summary allDay:(BOOL)shouldBeAllDay;
- (NSColor *)multidayViewDrawingColorForNewDraggingEvent:(CHMultidayView *)multidayView;
- (void)multidayView:(CHMultidayView *)multidayView eventsWereSelected:(NSArray *)events clickCount:(NSInteger)clickCount;
- (void)multidayView:(CHMultidayView *)multidayView modifyEvent:(id <CHCalendarEvent>)calendarEvent originalStartDate:(NSCalendarDate *)originalStartDate newStartDate:(NSCalendarDate *)newStartDate endDate:(NSCalendarDate *)newEndDate summary:(NSString *)newSummary;
- (void)multidayView:(CHMultidayView *)multidayView selectedDateDidChange:(NSDate *)newSelectedDate;
- (BOOL)multidayView:(CHMultidayView *)multidayView shouldAllowDragOfEvent:(id <CHCalendarEvent>)calendarEvent;
- (BOOL)multidayView:(CHMultidayView *)multidayView shouldHighlightEvent:(id <CHCalendarEvent>)calendarEvent;
- (NSString *)multidayView:(CHMultidayView *)multidayView toolTipForEvent:(id <CHCalendarEvent>)calendarEvent;
@end
#endif
