//
//  CHNibLoadingObject.h
//  CHAppKit
//
//  Created by Rod Schmidt on 11/29/05.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

/*
CHNibLoadingObject is a class that provides functionality for properly loading a nib file and
releasing its resources when they are no longer needed. If you have a class that needs to load a nib file,
but it is not a window controller you should make that class a subclass of CHNibLoadingObject. This will ensure
that the objects in the nib are properly released.
*/
@interface CHNibLoadingObject : NSObject 
{
	NSMutableArray*	_topLevelNibObjects;
}

// Loads a nib file named nibName from the same bundle the class is in. 
//	Returns YES if the nib was loaded. 
//	Returns NO if the nib could not be loaded and writes a message out to the console
- (BOOL)loadNib:(NSString *)nibName;
- (BOOL)loadNib:(NSString *)nibName inBundle:(NSBundle *)bundle;

// Releases all top-level objects. Note that this is automatically done when the object is
// released, so this only needs to be called if the object loads multiple nibs and we want
// to dynamically clean up memory.
-(void)unloadNib;

@end
