//
//  CHPreferencesWindowController.h
//  Preference
//
//  Created by Rod Schmidt on Tue Oct 21 2003.
//  Copyright (c) 2003 infiniteNIL Software. All rights reserved.
//

#import <AppKit/AppKit.h>

/*
A Windows controller for displaying preference panes. Provides a toolbar and
auto-resizes the window according to the size of the selected pane.
*/

@interface CHPreferencesWindowController : NSWindowController 
{
	IBOutlet NSView *	_expandedView;
	IBOutlet NSMatrix *	_expandedMatrix;
	
	// All the preference panes to be displayed
	NSMutableDictionary* _preferencePanes;
	
	// All the meta data about the preference panes
	NSMutableDictionary* _preferencePaneData;
	
	// The name of the currently selected/displayed preference pane
	NSString* _currentPane;
	
	// The pane that will be selected once the unselect notification is received
	NSString* _paneToSelect;
	
	// Should the panes be ordered alphabetically. Default is YES;
	BOOL _orderAlphabetically;
	
	// The array of pane names ordered by how the should be displayed.
	NSMutableArray* _paneNamesInDisplayOrder;
	
	// The string to use for the title of the window
	NSString* _windowTitle;
	
	BOOL _sentSelectMessage;
	
	// Do we display all the preference pane images or just in a toolbar
	BOOL _expandedMode;
	
	// In expanded mode, points to the view for the current pref pane
	NSView *_paneView;
}

// Initialize the window with the default width in expanded mode
-(id)initInExpandedMode;

// Initialize the window with a certain width and the mode
-(id)initWithWidth:(int)width expandedMode:(BOOL)expanded;

// Initialize the window with a certain width
-(id) initWithWidth: (int) width;

// Add a non bundled preference pane to the window.
//
// Parameters:
// aName		- a symbolic name for the preference pane
// aClass		- the controlling class for the preference pane
// imageName	- the name of the icon to use for the pane in the toolbar
// label		- The label for the icon on the toolbar
// toolTip		- The tooltip for the icon on the toolbar
- (void) addPreferencePaneNamed: (NSString*) aName ofClass: (Class) aClass 
	imageName: (NSString*) imageName label: (NSString*) label 
	toolTip: (NSString*) toolTip;

// Order the toolbar icons for the panes according to the order of the names in
// paneNames. By default the order is alphabetical.
- (void) setDisplayOrder: (NSArray*) paneNames;

// Set the title for the preferences window. If title is null the default will 
// be used. You can include %@ in the title and it will be replaced with the
// label for the currently selected preference pane.
- (void) setWindowTitle: (NSString*) title;

-(void) selectPaneNamed: (NSString*) name;

@end
