//
//  CHSoftwareUpdate.h
//  CHSoftwareUpdate
//
//  Created by Nick Zitzmann on Wed Mar 17 2004.
//  Copyright (c) 2004 Chronos LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "CHNibLoadingObject.h"

/* Here's how software update works:
1. At the location <http://www.chronosnet.com/versions.plist> we've stored a property list file containing version data on all of our products.
2. The plist contains one dictionary for each of our products. The dictionary is named after each product.
3. Each dictionary contains the following keys:
CurrentVersion (string) - contains the latest version number
ChangesFileURL (string) - contains a URL pointing to where the client can download a list of
						  changes
DownloadURL (string) - contains a URL pointing to where the user can download the file
FreeChangesFileURL (string) **NEW** - contains a URL pointing to where the client can download
									  a list of changes, intended for free updates of apps
MajorUpdatePrice (string) **NEW** - contains the price of a major upgrade; string must be
                                    formatted in the plist (e.g. "$25 USD")
ReleaseDate (date) **NEW** - contains the release date of the new version
MinimumOSVersion (number) - contains the minimum OS version required to run the software,
							must be written in hexadecimal (e.g. 0x0603 for 6.0.3, 0x1044 for
							10.4.4, 0x1100 for 11.0.0, etc.) and translated to decimal
*/

@interface CHSoftwareUpdate : CHNibLoadingObject
{
	// IB outlets
	IBOutlet NSImageView *_iconImg;
	IBOutlet NSTextField *_headerTxt;
	IBOutlet NSTextField *_versionNumberTxt;
	IBOutlet NSTextField *_releaseDateTxt;
	IBOutlet NSTextField *_upgradePriceTxt;
	IBOutlet NSTextField *_minimumOSTxt;
	IBOutlet NSTextView *_whatsNewTxt;
	IBOutlet NSWindow *_updateWindow;
	
	// Ivars
	BOOL _majorVersionChanged;
	BOOL _osVersionRequirementMet;
	
	// Others
	BOOL tellIfUpToDate;
	BOOL displayErrors;
	BOOL loadInBackground;
}
+ (CHSoftwareUpdate *)sharedInstance;

- (void)checkForUpdateUsingBundle:(NSBundle *)aBundle;
- (void)checkForUpdateUsingBundle:(NSBundle *)aBundle relativeToWindow:(NSWindow *)parentWindow;

- (void)setTellIfUpToDate:(BOOL)newValue;
- (void)setDisplayErrors:(BOOL)newValue;
- (void)setLoadInBackground:(BOOL)newValue;

- (IBAction)updateClicked:(id)sender;
- (IBAction)laterClicked:(id)sender;

- (NSString*)getLatestVersion:(NSBundle *)aBundle;

@end
