//
//  CHTabView.h
//  CHTabs
//
//  Created by Jerry Halls on Thu Nov 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <AppKit/AppKit.h>
#import <CHFoundation/CHFoundation.h>

@class CHTabViewItem;

typedef enum _CHTabViewType {
   CHTopTabs = 0,
   CHTopReversedTabs = 1,
   CHBottomTabs = 2
} CHTabViewType;


//================================================================================
//	CHTabView
//================================================================================
@interface CHTabView : NSView {
	CHTabViewType   _tabViewType;
	NSView			*_contentView;
	NSMutableArray  *_tabViewsArray;
	NSMutableArray  *_trackingRectsArray;
	NSMutableArray  *_closeButtonsArray;
	NSPopUpButton	*_hiddenTabsButton;
	float			_tabBarHeight;
	float			_lastTabRightMargin;
	
	float			_minTabWidth;
	float			_maxTabWidth;
	BOOL			_isTabBarVisible;
	BOOL			_alwaysShowTabBar;

	NSDictionary	*_dimmedAttributesDictionary;
	NSDictionary	*_attributesDictionary;
	NSDictionary	*_attributesDictionaryLeopardBezel;
	NSDictionary	*_dimmedAttributesDictionaryMetal;
	CGFloat			_labelHeight;
	id				_delegate;
	
	NSColor*		_tabColor;
	NSColor*		_tabBackgroundColor;
}

- (void)setTabViewColor:(NSColor*)aColor backgroundColor:(NSColor*)aBackgroundColor;
- (void)setTabViewType:(CHTabViewType)tabViewType;
- (void)setAlwaysShowTabBar:(BOOL)aAlwaysShowTabBar;
- (BOOL)alwaysShowTabBar;
- (void)setDelegate:(id)anObject;
- (id)delegate;

- (void)addTabViewItem:(CHTabViewItem*)aTabViewItem;
- (void)removeTabViewItem:(CHTabViewItem*)aTabViewItem;

- (NSInteger)numberOfTabViewItems;
- (CHTabViewItem *)tabViewItemAtIndex:(NSInteger)index;
- (NSArray*)tabViewItems;
- (int)indexOfTabViewItem:(CHTabViewItem*)aTabViewItem;

- (void)selectTabViewItemAtIndex:(NSInteger)aIndex;
- (void)selectFirstTabViewItem;
- (void)selectLastTabViewItem;
- (void)selectNextTabViewItem;
- (void)selectPreviousTabViewItem;
- (void)selectTabViewItem:(CHTabViewItem*)aTabViewItem;
- (CHTabViewItem*)selectedTabViewItem;
- (int)selectedTabViewItemIndex;

- (void)hiliteTabViewItemAtIndex:(int)aIndex;
- (void)unhiliteTabViewItemAtIndex:(int)aIndex;
- (CHTabViewItem*)hilitedTabViewItem;

- (void)drawTab:(NSInteger)aTabIndex;
- (void)refreshTabBar;

- (void)setMinTabWidth:(float)aWidth;
- (void)setMaxTabWidth:(float)aWidth;
- (CGFloat)calculateTabWidth;
- (NSInteger)calculateMaxNumVisibleTabs;
- (NSInteger)calculateNumVisibleTabs;
- (NSInteger)calculateNumHiddenTabs;
- (NSRect)calculateTabRectForTabViewItemAtIndex:(NSInteger)aTabIndex;
- (NSPoint)calculateTabBarPosition;
- (NSRect)calculateTabBarRect;
- (BOOL)calculateIsTabBarVisible;
- (float)tabBarHeight;

@end

//================================================================================
//	CHTabViewDelegate protocol
//================================================================================

@interface NSObject(CHTabViewDelegate)
- (BOOL)CHTabView:(CHTabView *)tabView shouldSelectTabViewItem:(CHTabViewItem *)tabViewItem;
- (void)CHTabView:(CHTabView *)tabView willSelectTabViewItem:(CHTabViewItem *)tabViewItem;
- (void)CHTabView:(CHTabView *)tabView didSelectTabViewItem:(CHTabViewItem *)tabViewItem;
- (void)CHTabViewDidChangeNumberOfTabViewItems:(CHTabView *)TabView;

- (void)CHTabView:(CHTabView *)tabView willRemoveTabViewItem:(CHTabViewItem *)tabViewItem;
@end

