//
//  CHDatabase+Calendars.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 06/13/2005.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "CHDatabase.h"

// Constants listed here because some other classes uses them
extern NSString * const DBCalendarTableName;
extern NSString * const DBCalendarEventTableName;
extern NSString * const DBCalendarTaskTableName;
extern NSString * const DBCalendarAlarmTableName;
extern NSString * const DBCalendarAttendeeTableName;
extern NSString * const DBCalendarRecurrenceTableName;
extern NSString * const DBCalendarOrganizerTableName;

@class PGCalendarRecord, PGCalendar, PGCalendarEvent, PGCalendarTask, PGCalendarEventTaskAttribute, PGCalendarAlarm, PGCalendarAttendee, PGRecurrence, PGCalendarOrganizer;
@class PGRecord;
@class PGCalendarsModel;

@interface CHDatabase (Calendars)
- (BOOL)createAllCalendarTables;

- (PGCalendarRecord *)getCalendarRecordWithUniqueID:(NSString *)uniqueID;

- (PGCalendar *)getCalendarWithUniqueID:(NSString *)uniqueID;
- (PGCalendar *)getCalendarWithRowID:(long long)rowID;
- (NSArray *)calendars;
- (NSArray *)calendarsWithoutChildren;
- (unsigned int)calendarsCount;
- (void)getChildrenOfCalendar:(PGCalendar *)calendar getChildrenOfChildren:(BOOL)getMoreChildren;
- (NSArray *)calendarOrder;
- (void)setCalendarOrder:(NSArray *)calendarOrder;

- (PGCalendarEvent *)getEventWithUniqueID:(NSString *)uniqueID;
- (PGCalendarEvent *)getEventWithRowID:(long long)rowID;
- (PGCalendarTask *)getTaskWithUniqueID:(NSString *)uniqueID;
- (PGCalendarTask *)getTaskWithRowID:(long long)rowID;
- (NSArray *)getEventsLinkedToContactRecord:(PGRecord *)contactRecord;
- (NSArray *)getTasksLinkedToContactRecord:(PGRecord *)contactRecord;
- (NSArray *)events;
- (NSArray *)eventsWithoutChildren;
- (unsigned int)eventsCount;
- (NSArray *)tasks;
- (NSArray *)tasksWithoutChildren;

- (PGCalendarAlarm *)getAlarmWithUniqueID:(NSString *)uniqueID;
- (PGCalendarAlarm *)getAlarmWithRowID:(long long)rowID;
- (PGCalendarAttendee *)getAttendeeWithUniqueID:(NSString *)uniqueID;
- (PGCalendarAttendee *)getAttendeeWithRowID:(long long)rowID;
- (PGRecurrence *)getRecurrenceWithUniqueID:(NSString *)uniqueID;
- (PGRecurrence *)getRecurrenceWithRowID:(long long)rowID;
- (PGCalendarOrganizer *)getOrganizerWithRowID:(long long)rowID;
- (NSArray *)calendarAlarms;
- (NSArray *)attendees;
- (NSArray *)recurrences;
- (NSArray *)organizers;

- (BOOL)addCalendarRecord:(PGCalendarRecord *)aRecord;
- (BOOL)updateCalendarRecord:(PGCalendarRecord *)aRecord;
- (BOOL)deleteCalendarRecord:(PGCalendarRecord *)aRecord;
- (void)updateVersionNumberInCalendarRecord:(PGCalendarRecord *)record;

- (BOOL)deleteAttendeesWithRecord:(PGRecord *)aRecord;	// for internal use only - called when we need to delete a contact or group record
- (NSArray *)executableAliasesInAlarmTable;	// returns NSArray of NDAlias objects
- (NSArray *)scriptAliasesInAlarmTable;	// returns NSArray of NDAlias objects

- (PGCalendarsModel *)calendarsModel;

- (BOOL)addColorAndIdentifierToCalendarTable;
- (BOOL)addTimeZonesToEventTaskAndAlarmTables;
- (BOOL)createOrganizerTableIfNecessary;
- (BOOL)addSubscriptionOptionsToCalendarTable;
@end
