//
//  CHDatabase+ContactsAndGroups.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 03/30/2005.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "CHDatabase.h"
#import <CHFoundation/CHFoundation.h>

#define CHDatabaseIterationsBetweenPoolReleases	20

@class PGRecord, PGPerson, PGGroup, PGMultiValue, PGSearchSession, PGContactsModel;

// Database table names, listed here because some other classes use them
extern NSString * const DBContactTableName;
extern NSString * const DBAddressTableName;
extern NSString * const DBPhoneTableName;
extern NSString * const DBEmailTableName;
extern NSString * const DBCustomFieldTableName;
extern NSString * const DBGroupTableName;
extern NSString * const DBGroupInclusionTableName;

// Database column names, listed here because some other classes use them
// Generic
extern NSString * const DBGenericRowID;
extern NSString * const DBGenericContactID;
extern NSString * const DBGenericGroupID;
extern NSString * const DBGenericLabel;
// DBContactTableName
extern NSString * const DBContactUniqueID;
extern NSString * const DBContactFirstName;
extern NSString * const DBContactMiddleName;
extern NSString * const DBContactLastName;
extern NSString * const DBContactJobTitle;
extern NSString * const DBContactDepartment;
extern NSString * const DBContactOrganization;
extern NSString * const DBContactTitle;
extern NSString * const DBContactSuffix;
extern NSString * const DBContactNotes;
// DBAddressTableName
extern NSString * const DBAddressStreet;
extern NSString * const DBAddressCity;
extern NSString * const DBAddressState;
extern NSString * const DBAddressPostalCode;
extern NSString * const DBAddressCountry;
// DBPhoneTableName
extern NSString * const DBPhoneNumber;
extern NSString * const DBPhoneRawNumber;
// DBEmailTableName
extern NSString * const DBEmailAddress;
// DBCustomFieldValue
extern NSString * const DBCustomFieldValue;
// DBGroupTableName
extern NSString * const DBGroupUniqueID;

@interface CHDatabase (ContactsAndGroups)
-(BOOL) createAllContactsAndGroupsTables;

- (PGRecord *)getRecordWithUniqueID:(NSString *)aUniqueID;
- (BOOL)isMember:(PGRecord *)aRecord;

- (PGPerson *)getPersonWithUniqueID:(NSString *)aUniqueID;
- (PGPerson *)getPersonWithRowID:(long long)aRowID;
- (PGPerson *)getAbridgedPersonWithUniqueID:(NSString *)aUniqueID;
- (PGPerson *)getAbridgedPersonWithRowID:(long long)aRowID;
- (PGPerson *)getPersonOwningMultiValueWithIdentifier:(NSString *)multiValueID multiValueProperty:(NSString **)mvProperty;

- (NSArray *)abridgedContactsLinkedToOrganization:(PGPerson *)anOrganization;
- (void)lookupAndLinkMatchingOrganizationToPerson:(PGPerson *)aPerson;
- (void)lookupAndLinkRelatedNameToPerson:(PGPerson *)aPerson relatedNameIndex:(unsigned int)index;
- (void)updateRelatedNameInContactsLinkedToPerson:(PGPerson *)aPerson;
- (void)updateRelatedNameLabelInContactsLinkedToPerson:(PGPerson *)aPerson;

- (PGGroup *)getGroupWithUniqueID:(NSString *)aUniqueID;
- (PGGroup *)getGroupWithRowID:(long long)aRowID;

- (NSArray *)people;
- (NSArray *)peopleFilteredUsingSearchSession:(PGSearchSession *)session;
- (NSArray *)abridgedPeople;
- (NSArray*)groups;
- (PGPerson*)me;
- (void)setMe:(PGPerson *)newMe;
- (unsigned int)peopleCount;
- (unsigned int)groupsCount;
- (CHLongLongIndexSet *)idsOfDuplicatesOfPeople:(NSArray *)peopleArray;
- (CHLongLongIndexSet *)idsOfDuplicateContactsAlreadyInDatabase;
- (NSSet *)labelsInDatabaseForMultiValueProperty:(NSString *)property;

- (BOOL)markContactRecord:(PGRecord *)aRecord;
- (BOOL)unmarkContactRecord:(PGRecord *)aRecord;
- (BOOL)addContactRecord:(PGRecord *)aRecord;
- (BOOL)updateContactRecord:(PGRecord *)aRecord;
- (BOOL)deleteContactRecord:(PGRecord *)aRecord;
- (void)updateVersionNumberInRecord:(PGRecord *)record;

- (NSArray *)listViewPeople;
- (NSArray *)listViewPeopleForGroup:(PGGroup *)group;
- (NSArray *)listViewPeopleFilteredUsingSearchSession:(PGSearchSession *)searchSession;
- (PGPerson *)getListViewPersonWithUniqueID:(NSString *)aUniqueID;
- (PGPerson *)getListViewPersonWithRowID:(long long)aRowID;
- (void)changeListViewValue:(id)newValue property:(NSString *)property contactID:(NSString *)contactID;

- (NSArray *)keywordsMasterList;
- (NSArray *)keywordsMasterListWithUniqueIDs;	// returns array of dictionaries; keys are @"keyword" and @"uniqueID"
- (NSString *)masterListKeywordWithUniqueID:(NSString *)uniqueID;
- (NSString *)addKeywordToMasterList:(NSString *)keyword;	// returns unique ID if successful, nil if not
- (void)replaceKeywordAtMasterListIndex:(unsigned int)index withKeyword:(NSString *)newKeyword;
- (BOOL)replaceKeywordWithUniqueID:(NSString *)uniqueID withKeyword:(NSString *)newKeyword;
- (void)removeKeywordsFromMasterList:(NSArray *)keywordsToRemove;
- (BOOL)removeMasterListKeywordWithUniqueID:(NSString *)uniqueID;

- (NSMutableArray *)autocompleteTableForPartialString:(NSString *)whereString property:(NSString *)property;
- (NSMutableArray *)autocompleteTableForPartialNameString:(NSString *)aString;

- (NSArray *)sortData;
- (void)setSortData:(NSArray *)newSortData;

- (PGContactsModel *)contactsModel;

- (BOOL)createSubgroupInclusionTable;
- (BOOL)createCreatedAndModifiedDatesInListViewAndFillIn;
- (BOOL)fixTimesInOtherDates;

@end
