//
//  CHDatabase+Folders.h
//  StickyBrainKit
//
//  Created by Rod Schmidt on 3/25/2005.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import "CHDatabase.h"
#import "SBFolder.h"
#import "SBSmartFolder.h"

// Folder Types
////////////////

extern NSString * const SBBookFolderTypeName;
extern NSString * const SBDailyNotesFolderTypeName;
extern NSString * const SBDefaultTrashFolderTypeName;
extern NSString * const SBIPodFolderTypeName;
extern NSString * const SBJournalFolderTypeName;
extern NSString * const SBLibraryFolderTypeName;
extern NSString * const SBPalmFolderTypeName;
extern NSString * const SBPasswordsFolderTypeName;
extern NSString * const SBReceiptsFolderTypeName;
extern NSString * const SBUnfiledFolderTypeName;
extern NSString * const SBSmartFolderTypeName;
extern NSString * const SBSerialNumbersFolderTypeName;

@interface CHDatabase (Folders)

// Default folder creation methods
////////////////////////////////////

-(BOOL) addDailyNotesFolderIcon;
-(BOOL) addDailyNotesFolderType;
-(BOOL) addDailyNotesFolder;
-(BOOL) addDefaultNotesFolderIcon;
-(BOOL) addFolderTypeWithName: (NSString*) name;
-(BOOL) addJournalFolderIcon;
-(BOOL) addJournalFolderType;
-(BOOL) addJournalFolder;
-(BOOL) addNewDefaultFolderIcons;
-(BOOL) addNewFolderIcons;
-(BOOL) addPalmFolder;
-(BOOL) addSmartFolderIcon;
-(BOOL) addSmartFolderType;
-(BOOL) addTrashFolder;
-(BOOL) addTrashFolderType;
-(BOOL) addTrashIcon;
-(BOOL) createDefaultFolderIcons;
-(BOOL) createDefaultFolderTypes;
-(BOOL) createDefaultFolders;
-(BOOL) createFlaggedSmartFolder;
-(BOOL) createSerialNumberFolder;
-(BOOL) createSpecialBook: (NSString*) aName iconType: (NSString*) iconType hotKey: (int) hotKey 
				modifiers: (long) modifiers action: (SBHotKeyAction) action include:(BOOL) includeInDock;
-(BOOL) createSpecialBook: (NSString*) aName folderType: (NSString*) folderType iconType: (NSString*) iconType 
				   hotKey: (int) hotKey modifiers: (long) modifiers action: (SBHotKeyAction) action 
				  include:(BOOL) includeInDock;
-(BOOL) createSpecialBooks;
-(void) fixFolderReferences;

// Folder Settings methods
///////////////////////////

-(BOOL) updateDefaultsForBook: (SBFolder*) book;

// Folder CRUD Methods
////////////////////////

// Returns row ID of new folder. 0 means failure
-(long long) addBook: (NSString*) bookName parent: (SBFolder*) parentFolder;
- (BOOL)addFolder:(SBFolder *)folder;
-(BOOL) addFolderRelationshipsWithFolderID: (long long) folderID parentID: (long long) parentID;
-(BOOL) addFolderRelationshipWithFolderID: (long long) folderID parentID: (long long) parentID;
-(BOOL) addFolderWithName: (NSString*) name folderType: (NSString*) type 
				   parent: (long long) parentID newFolderID: (long long*) newID;
-(BOOL)addSmartFolder: (SBSmartFolder *)folder;
-(NSArray*) allFolders;
-(NSArray*) arrayOfFolderIDsWithParent: (long long) parentFolderID;
-(NSArray*) arrayOfFoldersWithParent: (long long) parentFolderID;
-(NSArray*) bookIDsForFolder: (SBFolder*) folder;
-(BOOL) defaultBooksExist;
- (NSArray *)noteUUIDsForFolderID:(long long)folderID;
-(BOOL) deleteBookAndContents: (SBFolder*) folder 
			 progressDelegate: (id) delegate 
			 progressCallback: (SEL) progressCallbackSelector;
-(BOOL) deleteBook: (SBFolder*) book moveContentsTo: (SBFolder*) bookDest
  progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;
-(BOOL) deleteFoldersWithParent: (long long) parentID 
			   progressDelegate: (id) delegate 
			   progressCallback: (SEL) progressCallbackSelector;
-(BOOL) moveBook: (SBFolder*) book toFolder: (SBFolder*) folderDest moveContentsTo: (SBFolder*) notesDest 
progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;
-(SBFolder *) flaggedFolder;
-(BOOL) flaggedFolderExists;
// Returns an array of NSDictionary with _rowid, folderID, parentID
-(NSArray *)folderRelationships;
-(long long) folderTypeID: (NSString*) folderTypeName;
// Returns an array of NSDictionary with _rowid and name keys
-(NSArray *)folderTypes;
-(SBFolder*) folderWithID: (long long) bookID;
- (SBFolder *)folderWithUUID:(NSString *)uuid;
-(SBFolder*) folderWithType: (long long) folderTypeID;
-(long long) folderIDWithType: (long long) folderTypeID;
-(SBFolder*) folderWithName: (NSString*) name;
-(long long) folderIDWithName:(NSString *) name;

// Return array of folders IDs which the user has at least read access to. Don't include the trash folder
-(NSArray*) folderIDsInLibraryWithAccess;

// Load in all the folders with a certain parent. If parentID is 0 then all
// root folders are returned
-(NSArray*) foldersWithParent: (long long) parentID;
-(NSArray*) folderIDsWithParent: (long long) parentID;

-(SBFolder*) libraryFolder;
-(BOOL) moveFolder: (SBFolder*) folder to: (SBFolder*) destination;
-(BOOL) renameBook: (SBFolder*) book to: (NSString*) newName;
-(NSArray*) subBooks: (long long) parentFolderID;

-(BOOL)updateSmartFolder:(SBSmartFolder *)folder;
-(BOOL)addRule:(SBSmartFolderRule *)rule;
-(BOOL)updateRule:(SBSmartFolderRule *)rule;
-(BOOL)deleteRules:(NSArray *)rules;
- (NSArray *)smartFolderRules;
- (SBSmartFolderRule *)smartFolderRuleWithUUID:(NSString *)uuid;

// Icon methods
///////////////////

-(BOOL) addIcon: (NSImage*) image smallImage: (NSImage*) smallImage 
		   type: (NSString*) iconType;
-(BOOL) addIconAtPath: (NSString*) imagePath type: (NSString*) iconType;
-(BOOL) setIconForBookWithID: (long long) bookID iconType: (NSString*) iconType;
-(BOOL) updateIconForBook: (long long) folderID toIcon: (long long) iconID;
-(BOOL) updateAllNotesIcon;
- (BOOL)updateFolderTypeForFolder:(SBFolder *)folder;
- (BOOL)updateFolder:(SBFolder *)folder;	// super-method that updates everything having to do with folders
-(BOOL)updateIconForDefaultNotesFolder;

@end
