//
//  CHDatabase+Notes.h
//  StickyBrainKit
//
//  Created by Rod Schmidt on 3/25/2005.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import "CHDatabase.h"
#import <CHFoundation/CHFoundation.h>

typedef struct {
	BOOL searchTitles;
	BOOL searchContents;
	BOOL searchLabels;
	BOOL searchKeywords;
	BOOL ignoreCase;
	BOOL wholeWords;
	BOOL startsWith;
	BOOL searchChildren;
} SearchOptions;

@class SBNote;
@class SBFolder;
@class ABPerson;

@interface CHDatabase (Notes)

+(CHDatabase*) databaseForNote: (SBNote*) note;

-(BOOL) addNewNote: (SBNote*) note toBook: (long long) bookID;	// deprecated, use -addNewNote: instead
- (BOOL)addNewNote:(SBNote *)note;
- (NSArray *)allAbridgedNotes;
// Returns an array of dictionarys containing noteID and folderID
- (NSArray *)allNoteBooks;
-(BOOL) assignNote: (long long) noteID toBook: (long long) bookID;
-(NSArray*) bookIDsWithNoteID: (long long) noteID;
-(long long) noteIDForNoteBookID: (long long) noteBookRowID;
-(NSDictionary*) bookNamesForNotes: (NSArray*) notes 
						   inBooks: (NSArray*) bookIDs 
						 folderIDs: (NSMutableDictionary*) folderIDs;
-(NSDictionary*) bookNamesForNoteIDs: (CHLongLongIndexSet *) noteIDs folderIDs: (NSMutableDictionary*) folderIDs;
-(BOOL) decryptPrivateNotesWithPassword:(NSString *)oldPassword andEncryptWithNewPassword:(NSString *)newPassword;
-(BOOL) deleteNote: (SBNote*) note;
-(BOOL) deleteNoteWithID: (long long) noteID;

- (SBNote *)firstNoteCreatedAfterDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;
- (SBNote *)firstNoteCreatedBeforeDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;
- (SBNote *)firstNoteCreatedOnDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;

-(CHLongLongIndexSet *)IDsofNotesLinkedToContact:(id)contact;
-(BOOL) isHotKeyInUse: (short) keyCode modifiers: (long) modifiers;
-(NSDictionary*) linksForNotes: (NSArray*) notes InBooks: (NSArray*) bookIDs;
-(void) loadBookNamesAndFolderIDsForNotes: (NSArray*) notes;
-(NSData*) loadContentsForNote: (long long) noteID;
-(NSString*) loadSearchableContentsForNote: (long long) noteID;
-(BOOL) manualSortNeededForColumn: (NSString*) columnName;
-(NSString*) buildTrashFoldersClause: (long long) parentFolderID;
-(BOOL) note: (long long) noteID inBook: (long long) bookID;
-(BOOL) note: (SBNote*) note isInBook: (SBFolder*) book;
-(NSArray*) noteBookIDsWithNoteID: (long long) noteID;
-(long long) noteBooksIDWithNoteID: (long long) noteID bookID: (long long) bookID;
-(BOOL)noteIDAndFolderIDForNoteBookID:(long long)noteBookRowID noteID:(long long *)noteID folderID:(long long *)folderID;
-(NSString*) noteColumnList;
-(int) noteCountForFolder: (SBFolder*) folder;

-(NSString*) searchPhraseForColumn: (NSString*) columnName 
							  text: (NSString*) searchText 
						   options: (SearchOptions*) options 
						isTextBlob: (BOOL) isTextBlob;

// Contact may be nil to not filter by contact
-(NSArray*) notesForFolder: (SBFolder*) folder 
				forContact: (id) contact 
				  sortedBy: (NSString*) columnName 
				descending: (BOOL) descending
			  privateNotes: (BOOL) privateNotes
		  onlyFlaggedNotes: (BOOL) onlyFlaggedNotes;

/*
 Retrieve the notes assigned to a folder
 
 Parameters
 folder					The folder to which the notes are assigned
 contact				The contact the notes are assigned to, can be either an ABPerson or PGPerson contact. May be nil if no contact
 columnNames			The names of the database columns to load. Nil to get all columns
 sortedBy				The column to sort the notes by
 descending				Yes if the notes should be sorted in descending order
 subFolders				Yes if the notes in the child folders of folder should be read as well.
 includeCommunications	Yes if the communication log for the note should be loaded
 privateNotes			Yes if private notes should be loaded
 */
-(NSArray*) notesForFolder: (SBFolder*) folder 
				forContact: (id) contact
			   columnNames: (NSArray*) columnNames
				  sortedBy: (NSString*) columnName 
				descending: (BOOL) descending 
			  privateNotes: (BOOL) privateNotes
		 onlyFlaggedNotes: (BOOL) onlyFlaggedNotes
				subFolders: (BOOL) subFolders 
	 includeCommunications: (BOOL) includeCommunications;

-(NSArray*) notesForFolder: (SBFolder*) folder 
				forContact: (id) contact 
				  sortedBy: (NSString*) columnName 
				descending: (BOOL) descending 
			  privateNotes: (BOOL) privateNotes
		  onlyFlaggedNotes: (BOOL) onlyFlaggedNotes
				subFolders: (BOOL) subFolders;

-(NSString*) notesSubqueryForNotes: (NSArray*) notes inBooks: (NSArray*) bookIDs;
-(NSArray*) notesWithIDs: (CHLongLongIndexSet *) noteIDs;

-(NSArray*) notesForQuery: (NSString*) query 
				  inBooks: (NSArray*) bookIDs 
	includeCommunications: (BOOL) includeCommunications
	   includeFolderNames: (BOOL) includeFolderNames
		 includeLinkCount: (BOOL) includeLinkCount
			includeAlarms: (BOOL) includeAlarm;

/*
 Retrieve the notes assigned to a folder that contain some text
 
 Parameters
 searchText		The text to look for
 folder			The folder to which the notes are assigned
 contact			The contact the notes are assigned to, can be either an ABPerson or PGPerson contact. May be nil if no contact
 columnNames		The names of the database columns to load. Nil to get all columns
 sortedBy		The column to sort the notes by
 descending		Yes if the notes should be sorted in descending order
 searchOptions	Options for the search like case insensitive, etc.
 privateNotes	Yes if private notes should be loaded
 */
-(NSArray*) notesWithText: (NSString*) searchText 
				forFolder: (SBFolder*) folder 
			   forContact: (id) contact
			  columnNames: (NSArray*) columnNames
				 sortedBy: (NSString*) columnName 
			   descending: (BOOL) descending 
			 privateNotes: (BOOL) privateNotes
		 onlyFlaggedNotes: (BOOL) onlyFlaggedNotes
			searchOptions: (SearchOptions) options;

-(NSArray *) duplicateNotesForFolder:(SBFolder *) folder 
								keys:(NSArray *) keys 
							sortedBy:(NSString *) sortKey 
						  descending:(BOOL) descending 
						  subFolders:(BOOL) subFolders;

-(SBNote*) noteWithID: (long long) noteID;
-(SBNote *)noteWithUUID:(NSString *)uuid;
-(long long) noteIDForNotesUserID: (long long) notesUserID;
-(NSArray *)notesWithFormID:(long long)formID;
-(DBRecord*) recordForNote: (SBNote*) note;
-(DBRecord*) userRecordForRowID: (long long) notesUserID;
-(BOOL) saveRecord: (DBRecord*) record forNote: (SBNote*) aNote;
-(void) sortNotes: (NSMutableArray*) notes usingColumn: (NSString*) columnName descending: (BOOL) descending; 
-(void) updateModifiedDateAndModifiedBy: (SBNote*) note record: (DBRecord*) record;

-(BOOL) moveNote: (SBNote*) note fromBook: (SBFolder*) book toBookWithID: (long long) bookID;
-(BOOL) removeNote: (long long) noteID fromBook: (SBFolder*) book;
-(BOOL) updateColorForNote: (SBNote*) note;
-(BOOL) updateLabelForNote: (SBNote*) note;
-(BOOL) updateContentsForNote: (SBNote*) note;
-(BOOL) updateFloatingForNote: (SBNote*) note;
-(BOOL) updateFrameForNote: (SBNote*) note;
-(BOOL) updateHotKeyForNote: (SBNote*) note;
-(BOOL) updateMetaDataForNote: (SBNote*) note;
-(BOOL) updateUserAttributesForNote: (SBNote*) note;
-(BOOL) updateNote: (SBNote*) note;
// update the note, conditionally updating the contents
-(BOOL) updateNote: (SBNote*) note updateContents: (BOOL) updateContents;
-(BOOL) updateNote: (SBNote*) note updateContents: (BOOL) updateContents updateUserAttributes: (BOOL) updateUserAttributes;
- (BOOL)updateNote:(SBNote *)note updateContents:(BOOL)updateContents updateNoteAttributes:(BOOL)updateAttributes updateFolderRelationships:(BOOL)updateFolders;
-(BOOL) updateNoteAttributesForNote:(SBNote*)note;
-(BOOL) updateMarginSettingsForNote: (SBNote*) note;
-(BOOL) updateOpacityForNote: (SBNote*) note;
-(BOOL) updatePictureAndTextureForNote: (SBNote*) note;
-(BOOL) updatePrivateForNote: (SBNote*) note;
-(BOOL) updateFlaggedForNote: (SBNote*) note;
-(BOOL) updateSelectionForNote: (SBNote*) note;

// Load the latest version number for a note or retrieve it from record. Pass in nil for record and it will get the
// version from the database
-(void) updateVersionForNote: (SBNote*) aNote record: (DBRecord*) record;

-(BOOL) updateWindowTypeForNote: (SBNote*) note;

-(BOOL) markNote: (SBNote*) note;
-(void) unmarkNote: (SBNote*) note;

-(NSArray*) allNoteRecordsForSpotlight;
-(void) deleteSpotlightMetaData;

#ifdef USE_DENORMALIZED_SCHEMA
-(BOOL) addFolderToInFolders: (long long) folderID forNote: (long long) noteID;
-(BOOL) removeFolderFromInFolders: (long long) folderID forNote: (long long) noteID;
#endif

// Return an array of row ids of note meeting a certain condition.
// Params:
//	conditions should be of the same form you would pass to open base in a selectRecords...: call
-(NSArray*) rowIDsForNotesWhere: (NSString*) condition;

// Insert some new NotesUserRecords for the given user
-(BOOL) addNewNotesUserRecordsForUser: (long long) userRowID;

// get all the notes that have a shortcut
-(NSArray*) notesWithShortcuts;

// Remove quotes from UUID column in Notes Table. Our Palm conduit, at one time, had a bug that put quotes
// around the UUIDs. This method removes those quotes
-(BOOL) fixQuotedUUIDsInNotesTable;

@end
