//
//  CHDatabase+Privileges.h
//  StickyBrainKit
//
//  Created by Rod Schmidt on 3/25/2005.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import "CHDatabase.h"
#import "CHTypes.h"

@class SBUser, SBNote;

extern NSString* const CHDatabase_AllContacts;

@interface CHDatabase (Privileges)

-(BOOL) createChronosAdminUser;
-(BOOL) createDefaultUsers;

-(BOOL) initAdminPrivilegesForUser: (long long) adminUserID;
-(BOOL) initAdminPrivilegesForUserName: (NSString*) userName;

-(BOOL) initAdminCalendarPrivilegesForUser: (long long) adminUserID;
-(BOOL) initAdminCalendarPrivilegesForUserName: (NSString*) userName;

-(BOOL) initAdminContactPrivilegesForUser: (long long) adminUserID;
-(BOOL) initAdminContactPrivilegesForUserName: (NSString*) userName;

-(BOOL) updatePrivilege: (CHAccessLevel) level forFolder: (long long) folderID andUsers: (NSArray*) userIDs;
-(BOOL) updatePrivilege: (CHAccessLevel) level forContact: (NSString*) contactID andUsers: (NSArray*) userIDs;
-(BOOL) updatePrivilege: (CHAccessLevel) level forCalendarID: (long long) calendarID andUsers: (NSArray*) userIDs;

-(BOOL) deleteCalendarPrivilegesForCalendarWithRowID: (long long) calendarID;
-(BOOL) deletePrivilegesForFolder: (long long) folderID andUsers: (NSArray*) userIDs;

-(void) refreshFolderPrivilegesCache;
- (void)refreshCalendarPrivilegesCache;

// User methods
////////////////

-(CHAccessLevel) accessLevelForUser: (NSString*) userID andFolder: (long long) folderID;
-(CHAccessLevel) accessLevelForUser: (NSString*) userID andNote: (long long) noteID;
-(NSDictionary*) calendarAccessForUser: (long long) userID;
- (CHAccessLevel)calendarAccessForUserID:(long long)userID calendarID:(long long)calendarID;
- (CHAccessLevel)calendarAccessForUser:(NSString *)userID calendarID:(long long)calendarID;
-(NSDictionary*) calendarAccessForUsers: (NSArray*) userIDs;
-(NSDictionary*) contactAccessForUser: (long long) userID;
- (CHAccessLevel)allContactsAccessForUser:(long long)userID;
-(NSDictionary*) contactAccessForUsers: (NSArray*) userIDs;
-(NSDictionary*) folderAccessForUsers: (NSArray*) userIDs;
-(BOOL) markAllFolderPrivileges: (CHAccessLevel) level forFolders: (NSArray*) folders andUserIDs: (NSArray*) userIDs;
-(BOOL) markAllCalendarPrivileges: (CHAccessLevel) level forCalendars: (NSArray*) calendars andUserIDs: (NSArray*) userIDs;
-(BOOL) markAllContactPrivileges: (CHAccessLevel) level forContacts: (NSArray*) contacts andUserIDs: (NSArray*) userIDs;
-(long long) rowIDForUserID: (NSString*) userID;
-(NSArray*) rowIDsForCurrentUsers;
- (NSString *)userIDForRowID:(long long)rowID;
- (void)removeCachedUserWithRowID:(long long)rowID;	// this method is intended to be used only by the OB notification handler; you shouldn't need to call this anywhere else
-(void) setAccessLevelsForNote: (SBNote*) note usingFolderIDs: (NSArray*) folderIDs;

// User CRUD methods
//////////////////////

-(BOOL) addUser: (SBUser*) user;
-(BOOL) addUser: (SBUser*) user toGroup: (NSString*) groupName admin: (BOOL) isAdmin;
-(BOOL) deleteUser: (SBUser*) user;
-(BOOL) updateKey: (NSString*) key forUser: (SBUser*) user;
-(NSArray*) users;

// License methods
//////////////////////

-(int)figureNumUsersWithPrivilegesForProduct:(NSString*)aProductNameString;
-(int)figureNumUsersWithNotePrivileges;
-(int)figureNumUsersWithContactPrivileges;
-(int)figureNumUsersWithCalendarPrivileges;


@end
