//
//  CHOBBackup.h
//  CHOBBackup
//
//  Created by Nick Zitzmann on 08/13/2004.
//  Copyright 2004 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// Application ID of the backup preferences file:
extern NSString * const CHOBBackupPrefsID;

// Restore notifications
extern NSString * const CHOBBackupLocalDatabaseWillBackupNotification;
extern NSString * const CHOBBackupLocalDatabaseDidBackupNotification;
extern NSString * const CHOBBackupLocalDatabaseWillRestoreNotification;
extern NSString * const CHOBBackupLocalDatabaseDidRestoreNotification;
extern NSString * const CHOBBackupPreferenceDidChangeNotification;

// Keys to the dictionary returned by -getBackupDetailsFromDotMacAccount:
extern NSString * const CHOBBackupDotMacLastUserKey;		// NSString
extern NSString * const CHOBBackupDotMacBackupDateKey;		// NSDate
extern NSString * const CHOBBackupDotMacLastComputerKey;	// NSString

@class OpenBase, DMTransaction, SFAuthorization;

@interface CHOBBackup : NSObject
{
	OpenBase	*_db;
	BOOL		_preChronosName;
	SFAuthorization *_authorization;
}

+ (CHOBBackup *)sharedInstance;

- (void)setDatabaseToBackup:(OpenBase *)newDB;
- (OpenBase *)databaseToBackup;
- (NSString *)backupPath;
- (void)setBackupPath:(NSString *)newPath;
- (NSDate *)lastBackupDate;
- (int)maxNumberOfBackupFiles;
- (void)setMaxNumberOfBackupFiles:(int)maxNumber;

- (void)resetBackupPath;

- (void)setAuthorization:(SFAuthorization *)authorization;	// required for restore to work correctly for non-admin users

- (void)performBackupAndCleanup;	// this method is intended to be called by the scheduler to do an automatic backup; to do a manual backup use -performManualBackupToPath: instead
- (void)performManualBackupToPath:(NSString *)path;
- (void)restoreDatabaseAtPath:(NSString *)path hostPassword:(NSString *)hostPassword;
- (void)restoreDatabaseAtPath:(NSString *)path newName:(NSString *)aNewName hostPassword:(NSString *)hostPassword;
- (BOOL) restoredPreChronosDatabase;
- (void) fireLocalDatabaseWillRestoreDistributedNotification;
- (void) fireLocalDatabaseDidRestoreDistributedNotification;

- (BOOL)canBackupToOrRestoreFromDotMacAccount;
- (DMTransaction *)performBackupToDotMacAccount;	// backs up and begins download asynchronously; returns DMTransaction object so that the caller can tell how much data has been sent so far
- (NSDictionary *)getBackupDetailsFromDotMacAccount:(int*)aCredentials;
- (DMTransaction *)beginRestoreFromDotMacAccount:(SInt64 *)contentLength withCredentials:(int)aCredentials;	// starts downloading the data asynchronously; it's the caller's responsibility to call -restoreDatabaseAtPath: after the download

@end
