//
//  OpenBaseNotificationHandler.h
//  StickyBrainKit
//
//  Created by Rod Schmidt on 8/10/2004.
//  Copyright 2004 Chronos, LC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "CHDatabase.h"

typedef enum {
	CHDatabase_NoAction,
	CHDatabase_Insert,
	CHDatabase_Update,
	CHDatabase_Delete
} CHDatabaseAction;

@interface OpenBaseNotificationHandler : NSObject 
{
	BOOL					_connected;	// Are we connected to the database yet?
	NSMutableArray			*_alreadyHandledRows;	// Our fearless redundancy table
	NSRecursiveLock			*_alreadyHandledRowsLock;
	NSMutableSet			*_databases;	// Databases we already handle
	NSRecursiveLock			*_databasesLock;
	NSMutableArray			*_delegates;
	NSRecursiveLock			*_delegatesLock;
	NSDictionary			*_actionMap;
}

+(id) sharedHandler;

-(void) addDatabase: (CHDatabase*) db;
-(void) removeDatabase: (CHDatabase*) db;

- (void)registerRedundantNotificationForRowID:(long long)aRowID tableName:(NSString *)tableName databaseID:(NSString *)databaseID action:(CHDatabaseAction)action;
- (void)registerRedundantNotificationsForRowIDs:(NSArray *)rowIDs tableName:(NSString *)tableName databaseID:(NSString *)databaseID action:(CHDatabaseAction)action;	// rowIDs must be an NSArray of NSNumbers, and the NSNumbers must have a long long value

-(id) delegateForTable:(NSString *)table;
-(void) setDelegate: (id) aDelegate forTable:(NSString *)table;

@end

@interface NSObject (OpenBaseNotificationHandlerDelegate)
-(void) handleDatabaseNotificationWithAction: (CHDatabaseAction) action table: (NSString*) tableName rowid: (long long) rowid databaseID:(NSString *)databaseID;
@end
