//
//  PGAddressBook.h
//  CHDatabaseKit
//
//  Created by Jerry Halls on Tue Jan 27 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

extern NSString * const PGAddressBookDidConnectNotification;

@class PGRecord, PGCalendarRecord, PGPerson, PGGroup, PGCalendar, PGCalendarOccurrableRecord, PGCalendarEvent, PGCalendarTask, PGSearchSession, PGCalendarSearchSession;


@interface PGAddressBook : NSObject
{
	NSMetadataQuery *_eventSearch;
}
- (NSString*)name;
- (NSString *)hostName;
- (NSString *)addressBookID;
- (NSString *)persistentAddressBookID;
- (NSString *)userID;
- (int)accessLevelForUser;
- (int)userAccessLevelForCalendar:(PGCalendar *)calendar;
- (BOOL)isConnected;
- (BOOL)isLocalReplica;
- (BOOL)isAdmin;

- (BOOL)connect;
- (void)disconnect;
- (void)startNotificationFor:(id)observer;
- (void)removeNotificationFor:(id)observer;

- (PGPerson*)getPersonWithUniqueID:(NSString*)aUniqueID; // returns a full person
- (NSArray *)getPeopleWithUniqueIDs:(NSArray *)uniqueIDs;
- (NSArray *)getPeopleWithUniqueIDs:(NSArray *)uniqueIDs progressDelegate:(id)progressDelegate progressAction:(SEL)action;
- (PGPerson *)getPersonWithRowID:(long long)rowID;
- (NSArray *)getPeopleWithRowIDs:(CHLongLongIndexSet *)rowIDs progressDelegate:(id)progressDelegate progressAction:(SEL)progressAction;
- (PGGroup *)getGroupWithUniqueID:(NSString *)aUniqueID;
- (NSArray *)getListViewPeopleWithUniqueIDs:(NSArray *)uniqueIDs;
- (NSArray *)people;
- (NSArray *)groups;
- (NSArray *)calendars;
- (NSArray *)writableCalendars;

- (BOOL)addRecord:(PGRecord *)aPerson;
- (BOOL)updateRecord:(PGRecord *)aPerson;
- (BOOL)removeRecord:(PGRecord *)aPerson;
- (BOOL)addCalendarRecord:(PGCalendarRecord *)aRecord;
- (BOOL)addCalendarRecordAndChildren:(PGCalendarRecord *)aRecord;
- (BOOL)updateCalendarRecord:(PGCalendarRecord *)aRecord;
- (BOOL)removeCalendarRecord:(PGCalendarRecord *)aRecord;
- (BOOL)save;	// does not save in the background
- (BOOL)saveInBackground:(BOOL)background;
- (BOOL)isMember:(PGRecord *)aRecord;

- (BOOL)deleteAllSpotlightsForContactsEventsAndTasks;
- (BOOL)deleteUserSpotlightsForContactsEventsAndTasks;

- (BOOL)addSpotlightForPerson:(PGPerson *)person;
- (BOOL)deleteSpotlightForPersonRowID:(long long)rowID;
- (CHLongLongIndexSet *)deletedPersonIDsInSpotlight;
- (NSArray *)metadataDictionariesForObjects:(NSArray *)objects;	// this method is concrete
- (NSArray *)metadataDictionariesForPeopleNeedingUpdateInSpotlight;
- (NSArray *)metadataDictionariesForPeopleNotInSpotlight;
- (BOOL)updateSpotlightForPerson:(PGPerson *)person;

- (BOOL)addSpotlightForOccurrableRecord:(PGCalendarOccurrableRecord *)record;
- (BOOL)deleteSpotlightForOccurrableRecordRowID:(long long)recordRowID isEvent:(BOOL)isEvent;
- (CHLongLongIndexSet *)deletedEventIDsInSpotlight;
- (CHLongLongIndexSet *)deletedTaskIDsInSpotlight;
- (PGCalendarEvent *)eventWithUniqueID:(NSString *)uniqueID;
- (NSArray *)metadataDictionariesForEventsNeedingUpdateInSpotlight;
- (NSArray *)metadataDictionariesForTasksNeedingUpdateInSpotlight;
- (NSArray *)metadataDictionariesForEventsNotInSpotlight;
- (NSArray *)metadataDictionariesForTasksNotInSpotlight;
- (PGCalendarTask *)taskWithUniqueID:(NSString *)uniqueID;
- (BOOL)updateSpotlightForOccurrableRecord:(PGCalendarOccurrableRecord *)record;

- (NSArray *)eventSearchResultsFromSession:(PGCalendarSearchSession *)session;
- (NSArray *)listViewPeopleSearchResultsFromSession:(PGSearchSession *)session inGroup:(PGGroup *)group;
- (NSArray *)peopleSearchResultsFromSession:(PGSearchSession *)session inGroup:(PGGroup *)group;
- (NSArray *)taskSearchResultsFromSession:(PGCalendarSearchSession *)session;

@end
