//
//  PGAddressBookLocal.h
//  CHDatabaseKit
//
//  Created by Jerry Halls on Thu Jan 29 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PGAddressBook.h"
#import "PGAddressBookVersionResolver.h"

#define PGAddressBookLocalIterationsBetweenPoolReleases		20

extern NSString * const PGKeywordMasterListDidChangeNotification;

@class CHDatabase;
@class PGRecord, PGPerson, PGGroup, PGAllGroup, PGSearchSession;
@class CHQueue, SBFolder;

// Represents the Organizer database. All access to the data should go
// through this class
@interface PGAddressBookLocal : PGAddressBook
{
	CHDatabase *_database;
	BOOL _disconnectOnDealloc;
	
	NSMutableSet *_peopleToAdd;
	NSRecursiveLock *_peopleToAddLock;
	NSMutableSet *_groupsToAdd;
	NSRecursiveLock *_groupsToAddLock;
	NSMutableSet *_recordsToUpdate;
	NSRecursiveLock *_recordsToUpdateLock;
	NSMutableSet *_recordsToDelete;
	NSRecursiveLock *_recordsToDeleteLock;
	
	NSMutableSet *_calendarsToAdd;
	NSRecursiveLock *_calendarsToAddLock;
	NSMutableSet *_calendarOccurrableRecordsToAdd;
	NSRecursiveLock *_calendarOccurrableRecordsToAddLock;
	NSMutableSet *_calendarAttributesToAdd;
	NSRecursiveLock *_calendarAttributesToAddLock;
	NSMutableSet *_calendarRecordsToModify;
	NSRecursiveLock *_calendarRecordsToModifyLock;
	NSMutableSet *_calendarRecordsToDelete;
	NSRecursiveLock *_calendarRecordsToDeleteLock;
	
	CHQueue *_saveQueue;
	id <PGAddressBookVersionResolver> _versionResolver;
}
-(int) errorCode;
-(NSString*) errorMessage;
- (void)setDisconnectOnDealloc:(BOOL)disconnectOnDealloc;
- (unsigned int)numberOfAddressBooksWithSameName;

- (PGRecord *)getRecordWithUniqueID:(NSString *)aUniqueID;

-(PGPerson*)getPersonWithUniqueID:(NSString*)aUniqueID; // returns a full person
- (PGPerson *)getPersonWithRowID:(long long)aRowID;
- (PGPerson *)getAbridgedPersonWithUniqueID:(NSString *)aUniqueID;  // returns a person with only the ID and modified date, and the person's full name
- (PGPerson *)getAbridgedPersonWithRowID:(long long)aRowID;
- (NSArray *)getDuplicatesOfPeople:(NSArray *)people;
- (NSDictionary *)getDuplicatesOfPeopleAlreadyInDatabaseWithProgressDelegate:(id)delegate progressAction:(SEL)action;

- (NSArray *)abridgedContactsLinkedToOrganization:(PGPerson *)anOrganization;
- (void)tryLinkingOrganizationToPerson:(PGPerson *)aPerson;
- (void)tryUnlinkingOrganizationWithRowID:(long long)linkedOrganizationID fromPerson:(PGPerson *)aPerson;
- (void)tryLinkingRelatedNameToPerson:(PGPerson *)aPerson relatedNameIndex:(unsigned int)index;
- (void)tryUpdatingRelatedNameInContactsLinkedToPerson:(PGPerson *)aPerson;
- (void)tryUpdatingRelatedNameLabelInContactsLinkedToPerson:(PGPerson *)aPerson;
//- (void)tryDeletingAllLinksToContactRowID:(long long)rowID;

- (PGGroup *)getGroupWithUniqueID:(NSString *)aUniqueID;
- (PGGroup *)getGroupWithRowID:(long long)aRowID;

- (NSArray*)people; // WARNING: Slow method; don't call often
- (NSArray *)peopleFilteredUsingSearchSession:(PGSearchSession *)session;
- (NSArray *)abridgedPeople;	// returns PGPerson objects with only name & company details filled out
- (PGAllGroup *)allGroup;
- (NSArray*)groups;
- (NSArray *)topLevelGroups;
//- (NSArray *)membersOfGroup:(PGGroup *)aGroup filteredUsingSearchSession:(PGSearchSession *)session;
- (PGPerson*)me;
- (void)setMe:(PGPerson *)newMe;

- (NSArray *)listViewPeople;
- (NSArray *)listViewPeopleFilteredUsingSearchSession:(PGSearchSession *)searchSession;
- (PGPerson *)getListViewPersonWithRowID:(long long)aRowID;
- (void)changeListViewValue:(id)newValue property:(NSString *)property contactID:(NSString *)contactID;

- (NSArray *)keywordsMasterList;
- (void)addKeywordToMasterList:(NSString *)keyword;
- (void)replaceKeywordAtMasterListIndex:(unsigned int)index withKeyword:(NSString *)newKeyword;
- (void)removeKeywordsFromMasterList:(NSArray *)keywordsToRemove;

/*- (PGSearchSession *)searchSession;
- (void)setSearchSession:(PGSearchSession *)newSearchSession;*/
- (NSArray *)sortData;
- (void)setSortData:(NSArray *)newSortData;

- (unsigned int)calendarsCount;
- (void)setCalendarOrderUsingCalendars:(NSArray *)calendarsArray;

- (NSArray *)scriptAliases;
- (NSArray *)executableAliases;

- (NSArray *)notesForPerson:(PGPerson *)person sortedBy:(NSString *)sortColumn descending:(BOOL)sortDescending;

- (NSMutableArray *)autocompleteTableForPartialString:(NSString *)whereString property:(NSString *)property;
- (NSMutableArray *)autocompleteTableForPartialNameString:(NSString *)aString;

@end
