//
//  PGCalendar.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 06/13/2005.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHFoundation/CHFoundation.h>
#import "PGCalendarRecord.h"

typedef enum
{
	PGCalendarLocal = 0,
	PGCalendarURLSubscribe,
	PGCalendarURLPublish,
	PGCalendarCannotPublishOrSubscribe
} PGCalendarPublishSubscribeStatus;

enum	// values for the exclude mask
{
	PGCalendarExcludeTasks	= 1 << 1,
	PGCalendarExcludeAlarms	= 1 << 2
};

@class PGCalendarEvent, PGCalendarTask;

@interface PGCalendar : PGCalendarRecord <NSCoding, NSCopying>
{
	CHMutableLongLongIndexSet *_eventIDs;
	CHMutableLongLongIndexSet *_taskIDs;
	NSCalendar *_cal;	// DO NOT ARCHIVE; NSCalendar cannot be archived due to a bug in Mac OS X
	
	NSMutableArray *_unsavedEvents;
	NSMutableArray *_unsavedTasks;
}
- (NSString *)title;
- (void)setTitle:(NSString *)newTitle;

- (NSString *)notes;
- (void)setNotes:(NSString *)newNotes;

- (BOOL)isReadOnly;
- (void)setReadOnly:(BOOL)newReadOnly;

- (NSURL *)calendarURL;
- (void)setCalendarURL:(NSURL *)newCalendarURL;

- (NSString *)colorString;
- (void)setColorString:(NSString *)colorString;

- (NSCalendar *)nsCalendar;
- (void)setNSCalendar:(NSCalendar *)aCalendar;

// Publish and subscribe support:
- (PGCalendarPublishSubscribeStatus)publishSubscribeStatus;
- (void)setPublishSubscribeStatus:(PGCalendarPublishSubscribeStatus)status;
- (long)publishSubscribeExcludeMask;
- (void)setPublishSubscribeExcludeMask:(long)newMask;
- (long long)publishSubscribeRefreshInterval;	// a value of 0 means do not auto-refresh
- (void)setPublishSubscribeRefreshInterval:(long long)interval;
- (NSDate *)publishSubscribeLastRefreshDate;
- (void)setPublishSubscribeLastRefreshDate:(NSDate *)newDate;

- (BOOL)addEvent:(PGCalendarEvent *)event;
- (NSArray *)events;
- (BOOL)removeEvent:(PGCalendarEvent *)event;

- (BOOL)addTask:(PGCalendarTask *)task;
- (NSArray *)tasks;
- (BOOL)removeTask:(PGCalendarTask *)task;

- (NSArray *)eventsOccurringDuringDay:(NSDate *)day;
- (NSArray *)eventsOccurringBetweenStartTime:(NSDate *)startTime endTime:(NSDate *)endTime;
- (NSArray *)tasksIncompleteOrCompletedInDays:(int)days sinceDate:(NSDate *)sinceDate;
- (NSArray *)tasksOccurringDuringDay:(NSDate *)day;
- (NSArray *)tasksOccurringBetweenStartTime:(NSDate *)startTime endTime:(NSDate *)endTime;

@end
