//
//  PGCalendarEvent.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 06/13/2005.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHFoundation/CHFoundation.h>
#import "PGCalendarEnums.h"
#import "PGCalendarOccurrableRecord.h"

@class PGCalendar, PGCalendarAlarm, PGCalendarAttendee, PGRecurrence;

@interface PGCalendarEvent : PGCalendarOccurrableRecord <CHCalendarEvent, NSCoding, NSCopying>
{
	NSMutableArray *_exceptionDates;
	NSCalendarDate *_startDate;
	NSCalendarDate *_endDate;
	BOOL _isAllDay;
	
	NSMutableDictionary *_nextDatesForDate;
	NSMutableDictionary *_previousDatesForDate;
	
	NSDateComponents *_cachedStartDateComponents;
	NSCalendarDate *_cachedStartDateWithoutTime;
	NSCalendarDate *_cachedEndDateWithoutTime;
	NSArray *_cachedRecurrences;
	NSArray *_cachedDetachedOccurrences;
	PGCalendar *_cachedCalendar;
}
- (NSString *)eventDescription;
- (void)setEventDescription:(NSString *)newDescription;

- (NSString *)location;
- (void)setLocation:(NSString *)newLocation;

- (NSString *)summary;	// this is the "title" of the event
- (void)setSummary:(NSString *)newSummary;

- (PGCalendarClassification)classification;
- (void)setClassification:(PGCalendarClassification)newClassification;

- (PGCalendarEventStatus)status;
- (void)setStatus:(PGCalendarEventStatus)newStatus;

- (unsigned int)priority;
- (void)setPriority:(unsigned int)priority;

- (NSCalendarDate *)originalDate;
- (void)setOriginalDate:(NSCalendarDate *)newOriginalDate;

- (NSCalendarDate *)startDate;
- (void)setStartDate:(NSCalendarDate *)newStartDate;

- (NSCalendarDate *)endDate;
- (void)setEndDate:(NSCalendarDate *)newEndDate;

- (BOOL)isAllDay;
- (void)setAllDay:(BOOL)newIsAllDay;

- (NSArray *)exceptionDates;
- (void)setExceptionDates:(NSArray *)overrideExceptionDates;
- (void)addExceptionDate:(NSCalendarDate *)exceptionDate;
- (void)removeExceptionDate:(NSCalendarDate *)exceptionDate;

- (NSURL *)url;
- (void)setURL:(NSURL *)newURL;

- (NSDate *)absoluteEndDate;	// returns the last date on which this event will occur adjusting for recurrences; returns nil if this event will recur forever
- (void)clearOccurrenceCaches;
- (BOOL)eventRecursOnOrBetweenStartDate:(NSDate *)startDate endDate:(NSDate *)endDate;
- (BOOL)eventRecursOnOrBetweenStartDate:(NSDate *)startDate endDate:(NSDate *)endDate countMultipleDayspan:(BOOL)countMultipleDays;	// if YES, then we'll return YES if the event spans multiple days and startDate is a time after the event's day but before the event ends, NO if not
- (NSCalendarDate *)nextOccurrenceOfEventAfterDate:(NSDate *)date;
- (NSCalendarDate *)previousOccurrenceOfEventBeforeDate:(NSDate *)date;

@end
