//
//  PGCalendarsModel.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 3/24/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class CHDatabase;
@class PGCalendarRecord, PGCalendar, PGCalendarEvent, PGCalendarTask, PGCalendarAlarm, PGCalendarAttendee, PGRecurrence;
@class CHLongLongIndexSet;

extern NSString * const PGCalendarsModelWasChangedNotification;
extern NSString * const PGCalendarsModelAddedRecordsKey;
extern NSString * const PGCalendarsModelModifiedRecordsKey;
extern NSString * const PGCalendarsModelDeletedRecordsKey;

@interface PGCalendarsModel : NSObject
{
	CHDatabase *_database;
	
	NSMutableArray *_calendars;
	NSMutableDictionary *_calendarsKeyedByUniqueID;
	NSMutableDictionary *_calendarsKeyedByRowID;
	NSConditionLock *_calendarsLock;
	
	NSMutableArray *_events;
	NSMutableDictionary *_eventsKeyedByUniqueID;
	NSMutableDictionary *_eventsKeyedByRowID;
	NSConditionLock *_eventsLock;
	
	NSMutableArray *_tasks;
	NSMutableDictionary *_tasksKeyedByUniqueID;
	NSMutableDictionary *_tasksKeyedByRowID;
	NSConditionLock *_tasksLock;
	
	NSMutableArray *_alarms;
	NSMutableDictionary *_alarmsKeyedByUniqueID;
	NSMutableDictionary *_alarmsKeyedByRowID;
	NSConditionLock *_alarmsLock;
	
	NSMutableArray *_attendees;
	NSMutableDictionary *_attendeesKeyedByUniqueID;
	NSMutableDictionary *_attendeesKeyedByRowID;
	NSConditionLock *_attendeesLock;
	
	NSMutableArray *_recurrences;
	NSMutableDictionary *_recurrencesKeyedByUniqueID;
	NSMutableDictionary *_recurrencesKeyedByRowID;
	NSConditionLock *_recurrencesLock;
	
	NSZone *_modelTempObjectZone;
}
- (id)initWithDatabase:(CHDatabase *)database;

- (CHDatabase *)database;

- (PGCalendarRecord *)calendarRecordWithUniqueID:(NSString *)uniqueID;

- (NSArray *)calendars;
- (PGCalendar *)calendarWithRowID:(long long)rowID;
- (NSArray *)calendarsWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGCalendar *)calendarWithUniqueID:(NSString *)uniqueID;

- (NSArray *)events;
- (PGCalendarEvent *)eventWithRowID:(long long)rowID;
- (NSArray *)eventsWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGCalendarEvent *)eventWithUniqueID:(NSString *)uniqueID;
- (NSArray *)eventsWithUniqueIDs:(NSArray *)uniqueIDs;

- (NSArray *)tasks;
- (PGCalendarTask *)taskWithRowID:(long long)rowID;
- (NSArray *)tasksWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGCalendarTask *)taskWithUniqueID:(NSString *)uniqueID;
- (NSArray *)tasksWithUniqueIDs:(NSArray *)uniqueIDs;

- (NSArray *)alarms;
- (PGCalendarAlarm *)alarmWithRowID:(long long)rowID;
- (NSArray *)alarmsWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGCalendarAlarm *)alarmWithUniqueID:(NSString *)uniqueID;

- (NSArray *)attendees;
- (PGCalendarAttendee *)attendeeWithRowID:(long long)rowID;
- (NSArray *)attendeesWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGCalendarAttendee *)attendeeWithUniqueID:(NSString *)uniqueID;

- (NSArray *)recurrences;
- (PGRecurrence *)recurrenceWithRowID:(long long)rowID;
- (NSArray *)recurrencesWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGRecurrence *)recurrenceWithUniqueID:(NSString *)uniqueID;

- (void)recheckCalendarsAgainstPrivileges;
- (void)resetAll;	// convenience method that calls all of the below
- (void)resetCalendars;
- (void)resetEvents;
- (void)resetTasks;
- (void)resetAlarms;
- (void)resetAttendees;
- (void)resetRecurrences;

+ (NSArray *)colorStringsForCalendars;
- (NSString *)nextAvailableCalendarColorString;

@end
