//
//  PGContactsModel.h
//  CHDatabaseKit
//
//  Created by Nick Zitzmann on 1/23/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CHFoundation/CHFoundation.h>

@class CHDatabase, PGPerson, PGGroup, PGAllGroup, PGSearchSession;

extern NSString * const PGContactsModelWasChangedNotification;

@interface PGContactsModel : NSObject
{
	CHDatabase *_database;
	
	PGAllGroup *_allGroup;
	NSMutableArray *_groups;
	NSMutableDictionary *_groupsKeyedByRowID;
	NSMutableDictionary *_groupsKeyedByUniqueID;
	NSMutableArray *_topLevelGroups;
	NSConditionLock *_groupsLock;
	
	NSMutableArray *_people;
	NSMutableDictionary *_peopleKeyedByRowID;
	NSMutableDictionary *_peopleKeyedByUniqueID;
	NSMutableDictionary *_peopleSearchResultsKeyedBySearchSession;
	NSConditionLock *_peopleLock;
	
	NSMutableArray *_listViewPeople;
	NSMutableDictionary *_listViewPeopleKeyedByRowID;
	NSMutableDictionary *_listViewPeopleKeyedByUniqueID;
	NSMutableDictionary *_listViewPeopleSearchResultsKeyedBySearchSession;
	NSConditionLock *_listViewPeopleLock;
	NSArray *_lastSortDataForListView;
	
	unsigned int _meIndex;
}
- (id)initWithDatabase:(CHDatabase *)db;
- (CHDatabase *)database;

- (NSArray *)people;
- (NSArray *)peopleMatchingSearchSession:(PGSearchSession *)searchSession;
- (PGPerson *)personWithRowID:(long long)rowID unabridgeIfNecessary:(BOOL)shouldUnabridge;
- (NSArray *)peopleWithRowIDs:(CHLongLongIndexSet *)rowIDs unabridgeIfNecessary:(BOOL)shouldUnabridge;
- (PGPerson *)personWithUniqueID:(NSString *)uniqueID unabridgeIfNecessary:(BOOL)shouldUnabridge;
- (NSArray *)peopleWithUniqueIDs:(NSArray *)uniqueIDs unabridgeIfNecessary:(BOOL)shouldUnabridge;

- (NSArray *)listViewPeople;
- (NSArray *)listViewPeopleMatchingSearchSession:(PGSearchSession *)searchSession;
- (PGPerson *)listViewPersonWithRowID:(long long)rowID;
- (NSArray *)listViewPeopleWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGPerson *)listViewPersonWithUniqueID:(NSString *)uniqueID;
- (NSArray *)listViewPeopleWithUniqueIDs:(NSArray *)uniqueIDs;

- (PGAllGroup *)allGroup;
- (NSArray *)groups;
- (PGGroup *)groupWithRowID:(long long)rowID;
- (NSArray *)groupsWithRowIDs:(CHLongLongIndexSet *)rowIDs;
- (PGGroup *)groupWithUniqueID:(NSString *)uniqueID;
- (NSArray *)groupsWithUniqueIDs:(NSArray *)uniqueIDs;
- (NSArray *)topLevelGroups;

- (void)resetAll;
- (void)resetGroups;
- (void)resetPeople;

@end

NSComparisonResult PGCMSortListView(PGPerson *person1, PGPerson *person2, void *context);
