//
//  PGGroup.h
//  Organizer
//
//  Created by Jerry Halls on Tue Jan 27 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CHFoundation/CHFoundation.h>
#import "PGRecord.h"

@class PGPerson;

@interface PGGroup : PGRecord <NSCoding, NSCopying>
{
	CHMutableLongLongIndexSet *_memberIDs;
	CHMutableLongLongIndexSet *_subgroupIDs;
	CHMutableLongLongIndexSet *_parentGroupIDs;
	NSMutableArray *_cachedMembers;
	
    NSMutableArray *_unsavedMembers;
    NSMutableArray *_unsavedSubgroups;
	NSMutableArray *_unsavedParentGroups;
}
- (NSArray *)members;
- (NSArray *)listViewMembers;
    // Returns an array of PGPerson.
    // Returns an empty array if this group doesn't contain any people

- (BOOL)addMember:(PGPerson *)person;
    // Adds person to this group. Does nothing if person is already part of this group (returns NO)
    // Raises if person is nil
    // Returns YES if successful

- (BOOL)removeMember:(PGPerson *)person;
    // Removes person from this group. Does nothing if person is not part of this group (returns NO)
    // Raises if person is nil
    // Returns YES if successful

- (NSArray *)subgroups;
    // Returns an array of PGGroup
    // Returns an empty array if this group doesn't contain any other groups

- (BOOL)addSubgroup:(PGGroup *)group;
    // Add group to this group. Does nothing if group is already part of this group (returns NO)
    // Recursions are not allowed (returns NO)
    // Raises if group is nil
    // Returns YES if successful

- (BOOL)removeSubgroup:(PGGroup *)group;
    // Removes group from this group. Does nothing if group is not part of this group (returns NO)
    // Raises if group is nil
    // Returns YES if successful

- (NSArray *)parentGroups;
    // Returns an array of PGGroup this group belongs to.
    // Returns an empty array if this group doesn't belong to any groups.

- (BOOL)setDistributionIdentifier:(NSString *)identifier forProperty:(NSString *)property person:(PGPerson *)person;
    // Allows groups to be used as distribution lists (mailing list in the case of the email property)
    // Example: For group "Football team" I want to use John's home email and Paul's work email
    // The default distribution identifier is the property's primary identifier
    // Use nil for the identifer to reset the distribution identifier to the default (e.g. primary)
    // Raises if property or person is nil
    // Returns YES is successful

- (NSString *)distributionIdentifierForProperty:(NSString *)property person:(PGPerson *)person;
    // Returns the distribution identifier for a given property and person. If not set then returns the property primary identifier
    // Raises if property or person is nil
    // Returns the distribution identifier or nil if not sucessfull

@end
