//
//  PGMultiValue.h
//  Organizer
//
//  Created by Nick Zitzmann on Thu Mar 25 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>


@interface PGMultiValue : NSObject <NSCoding, NSCopying>
{
	NSMutableArray *_labels;
	NSMutableArray *_values;
	NSMutableArray *_identifiers;	// note: we cannot tie identifiers to the database because we need to generate them in memory at add-time, rather than generating them when they've been added to the database
	NSString *_primaryIdentifier;
	NSMutableArray *_rowIDs;
}
// Accessors
- (unsigned)count;
- (NSArray *)identifiers;
- (id)valueForLabel:(NSString *)label;
- (id)valueAtIndex:(unsigned int)index;
- (NSString *)labelAtIndex:(unsigned int)index;
- (NSString *)identifierAtIndex:(unsigned int)index;
- (unsigned int)indexForIdentifier:(NSString *)identifier;  // returns NSNotFound if "identifier" is not present in the multi-value
- (NSString *)primaryIdentifier;	// returns nil if the multi-value is empty
- (unsigned int)indexOfPrimaryIdentifier;   // shortcut that returns the multi-value's index of the primary identifier, or NSNotFound if the multi-value is empty
- (id)valueForPrimaryIdentifier;	// shortcut that returns the value stored in the identifier marked as primary, returns nil if multi-value is empty
- (BOOL)containsValue:(id)aValue;
- (unsigned int)indexOfValue:(id)aValue;
- (unsigned int)indexOfRowID:(long long)rowID;
- (unsigned int)numberOfValuesWithLabel:(NSString *)label;
- (unsigned int)indexOfFirstInstanceOfLabel:(NSString *)label;
- (unsigned int)indexOfFirstInstanceOfValue:(id)value;

// Mutable stuff (this class is fully mutable)
- (NSString *)addValue:(id)value withLabel:(NSString *)label;
- (NSString *)insertValue:(id)value withLabel:(NSString *)label atIndex:(unsigned int)index;
- (void)removeAllValuesAndLabels;
- (void)removeValueAndLabelAtIndex:(unsigned int)index;
- (BOOL)replaceValueAtIndex:(unsigned int)index withValue:(id)value;
- (BOOL)replaceLabelAtIndex:(unsigned int)index withLabel:(NSString *)label;
- (BOOL)setPrimaryIdentifier:(NSString *)newPrimaryIdentifier;  // returns YES if the primary identifier was successfully set and NO if not
- (void)setValueForPrimaryIdentifier:(id)newValue;  // sets "newValue" as the new value of the value that is the primary identifier; if there is no primary identifier then a new value is filled in
- (void)moveValueAndLabelAtIndex:(unsigned int)index1 toIndex:(unsigned int)index2;

@end
