//
//  PGSearchCriteria.h
//  Organizer
//
//  Created by Nick Zitzmann on Wed Jul 28 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PGSearchEnums.h"

extern NSString * const PGSearchSessionDidUpdateNotification;
extern NSString * const PGSearchSessionDidFinishNotification;

@class PGGroup, PGPerson;

@interface PGSearchSession : NSObject <NSCopying, NSCoding>
{
	PGSearchLogic _logic;
	NSMutableArray *_criteria;
	NSMutableArray *_verbs;
	NSMutableArray *_operatorTypes;
	NSMutableArray *_searchItems;
	PGGroup *_group;
	NSMetadataQuery *_query;
	NSSet *_cachedSearchResults;
	unsigned int _options;
}
+ (PGSearchSession *)searchSessionWithLogic:(PGSearchLogic)someLogic criteria:(PGSearchCriteria)aCriteria verb:(PGSearchVerb)aVerb item:(NSString *)anItem; // this method creates a direct database search session
+ (PGSearchSession *)searchSessionWithLogic:(PGSearchLogic)someLogic criteria:(PGSearchCriteria)aCriteria operatorType:(NSPredicateOperatorType)anOperatorType item:(id)anItem;	// this method creates a Spotlight search session

- (PGSearchLogic)logic;
- (void)setLogic:(PGSearchLogic)newLogic;
- (void)addCriteria:(PGSearchCriteria)newCriteria verb:(PGSearchVerb)newVerb item:(NSString *)newItem;  // we're one-way mutable; at this time there is no need for a remove method
- (void)addCriteria:(PGSearchCriteria)newCriteria operatorType:(NSPredicateOperatorType)operatorType item:(id)newItem;
- (unsigned)count;
- (NSArray *)criteriaArray;
- (PGSearchCriteria)criteriaAtIndex:(unsigned)index;
- (NSArray *)verbArray;
- (PGSearchVerb)verbAtIndex:(unsigned)index;
- (NSArray *)operatorTypeArray;
- (NSPredicateOperatorType)operatorTypeAtIndex:(unsigned)index;
- (NSArray *)itemArray;
- (id)itemAtIndex:(unsigned)index;
- (PGGroup *)groupToSearch;
- (void)setGroupToSearch:(PGGroup *)aGroup;
- (unsigned int)options;
- (void)setOptions:(unsigned int)options;

// For search types containing verbs:
- (NSString *)sqlSearchString;
- (NSString *)sqlSearchStringWithColumns:(NSString *)columns;
- (BOOL)isPersonInSearchResults:(PGPerson *)aPerson;

// For search types containing operator types:
- (NSPredicate *)predicateForSpotlight;
- (BOOL)startQuery;
- (NSSet *)uniqueIDsOfFoundPeople;
- (BOOL)isQuerying;

@end
