//
//  SBFolder.h
//  StickyBrain3
//
//  Created by Rod Schmidt on Wed Jan 14 2004.
//  Copyright (c) 2004 Chronos, LC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "CHTypes.h"
#import "SBNote.h"

@class SBIcon, SBLibrary;

#define SBFolderTypeExternalLibrary		99	// A library in another database either local or remote. It's not in the
											// FolderTypes table

@interface SBFolder : NSObject <NSCoding>
{
	NSString*		_uuid;
	long long		_rowid;
	NSString*		_name;
	long long		_iconID;
	SBIcon			*_icon;
	long long		_folderType;
	NSMutableArray* _children;
	SBFolder*		_parent;
	long long		_parentID;
	NSArray*		_lastSelectedNoteIDs;
	int				_notesCount;
	BOOL			_expanded;
	NSString*		_dbID;
	NSString*		_dbUserID;
	NSPoint			_lastScrollPosition;

	int				_hotKey;
	long			_hotKeyModifiers;
	SBHotKeyAction  _hotKeyAction;
	BOOL			_includeInDock;
	
	// Note Defaults
	long long		_defaultNoteColorID;
	SBWindowType	_defaultWindowType;
	long long		_defaultFormID;
	NSFont*			_defaultNoteFont;
	NSSize			_defaultNoteSize;
	NSString*		_defaultTexture;
	NSString*		_defaultPicture;
	BOOL			_defaultPrivate;
	BOOL			_defaultFloating;
	float			_defaultOpacity;
}

+(BOOL) isValidFolderName: (NSString*) aName;
+(id) folderWithName: (NSString*) name databaseID: (NSString*) dbID userID: (NSString*) userID;
-(id) initWithName: (NSString*) name databaseID: (NSString*) dbID userID: (NSString*) userID;

-(void) applyDefaultsToNote: (SBNote*) note;

-(NSComparisonResult) compareNames: (SBFolder*) aFolder;
-(void)takeValuesFrom:(SBFolder *)aFolder;

-(NSString *)accessLevelKey;

-(NSString*) uuid;
-(void) setUUID: (NSString*) uuid;

-(long long) folderID;
-(void) setFolderID: (long long) newID;
-(NSString*) name;
-(void) setName: (NSString*) aName;
-(NSString*) databaseID;
-(NSString*) userID;
-(SBLibrary *)library;

-(long long) folderType;
-(void) setFolderType: (long long) type;

-(long long)iconID;
-(void)setIconID: (long long)iconID;

- (SBIcon *)icon;

- (NSSet *)allChildren;
-(int) childrenCount;
-(SBFolder*) childFolderAtIndex: (int) index;
-(void) addChildFolder: (SBFolder*) child;
-(void) insertChild: (SBFolder*) child atIndex: (int) index;
-(void) insertChildInOrder: (SBFolder*) child;
-(void) insertChildInOrder: (SBFolder*) child startingIndex: (unsigned) startingIndex;
-(NSArray*) children;
-(void) removeChildFolder: (SBFolder*) child;
-(void) removeAllChildren;
-(unsigned int) indexOfChild: (SBFolder*) child;

-(int) insertPositionForChild: (SBFolder*) child startingIndex: (unsigned) startingIndex;

-(BOOL) isLibrary;
-(BOOL) isExternalLibrary;
-(BOOL) isBook;
-(BOOL) isTrash;
-(BOOL) isInTrash;
-(BOOL) isInFolder: (SBFolder*) aFolder;
-(BOOL) isDailyNotesFolder;
-(BOOL) isJournalFolder;
-(BOOL) isSmartFolder;

-(SBFolder*) parent;
-(long long) parentID;
-(void) setParent: (SBFolder*) parent;
-(void) setParentID: (long long) parentID;

- (void)setIsExpanded:(BOOL)expanded;
- (BOOL)isExpanded;

-(SBFolder*) childFolderWithID: (NSNumber*) folderID andDatabaseID: (NSString*) databaseID userID: (NSString*) userID;
-(SBFolder*) childFolderWithName: (NSString*) name andDatabaseID: (NSString*) databaseID userID: (NSString*) userID;
-(SBFolder*) childFolderWithType: (long long) folderTypeID andDatabaseID: (NSString*) databaseID userID: (NSString*) userID;
-(SBFolder*) childFolderWithValue: (id) value forKey: (NSString*) key andDatabaseID: (NSString*) databaseID 
						   userID: (NSString*) userID;
-(NSArray *) childFoldersNotWithValue:(id) value
							   forKey:(NSString *) key
						andDatabaseID:(NSString *) databaseID 
							   userID:(NSString *) userID;

-(NSArray*) lastSelectedNoteIDs;
-(void) setLastSelectedNotes: (NSArray*) notes;
-(void) setLastSelectedNoteIDs: (NSArray*) noteIDs;
-(NSPoint) lastScrollPosition;
-(void) setLastScrollPosition: (NSPoint) position;

-(int) notesCount;
-(void) setNotesCount: (int) count;
-(void) incrementNoteCount;
-(void) addToNotesCount: (int) additionalNotes;

-(int) hotKey;
-(long) hotKeyModifiers;
-(SBHotKeyAction) hotKeyAction;
-(BOOL) includeInDock;
-(void) setHotKey: (int) hotKey modifiers: (long) modifiers action: (SBHotKeyAction) action include:(BOOL) includeInDock;

-(long long) defaultNoteColorID;
-(void) setDefaultNoteColorID: (long long) colorID;
-(SBWindowType) defaultWindowType;
-(void) setDefaultWindowType: (SBWindowType) windowType;
-(long long)defaultFormID;
-(void)setDefaultFormID:(long long)formID;
-(NSFont*) defaultNoteFont;
-(void) setDefaultNoteFont: (NSFont*) font;
-(NSSize) defaultNoteSize;
-(void) setDefaultNoteSize: (NSSize) size;
-(NSString*) defaultTexture;
-(void) setDefaultTexture: (NSString*) textureName;
-(NSString*) defaultPicture;
-(void) setDefaultPicture: (NSString*) pictureName;
-(BOOL) defaultPrivate;
-(void) setDefaultPrivate: (BOOL) isPrivate;

-(BOOL) defaultFloating;
-(void) setDefaultFloating: (BOOL) floating;
-(float) defaultOpacity;
-(void) setDefaultOpacity: (float) opacity;

-(void) incrementNoteCountByAddingNote: (SBNote*) aNote;
-(void) decrementNoteCountByRemovingNote: (SBNote*) aNote;

@end
