//
//  SBLibrary+Notes.h
//  CHDatabaseKit
//
//  Created by Rod Schmidt on 10/24/06.
//  Copyright 2006 Chronos, LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "SBLibrary.h"

@class SBSmartFolder;

@interface SBLibrary (Notes)

// Notes
////////

-(BOOL)addNewNote:(SBNote *)aNote;
-(BOOL)addNewNote:(SBNote *)note toBook: (SBFolder *)book;
-(BOOL)addNewNotes:(NSArray *)notes toFolder:(SBFolder *)folder progressDelegate:(id)delegate progressCallback:(SEL)progressCallbackSelector;
-(BOOL)addNotes:(NSArray *)notes toBookWithID: (long long)bookID progressDelegate: (id)delegate progressCallback: (SEL)progressCallbackSelector;

-(NSArray *)bookIDsWithNoteID: (long long)noteID;
-(NSDictionary *)bookNamesForNoteIDs:(CHLongLongIndexSet *)noteIDs folderIDs: (NSMutableDictionary *)folderIDs;
-(NSDictionary *)bookNamesForNotes: (NSArray *)notes 
						   inBooks: (NSArray *)bookIDs 
						 folderIDs: (NSMutableDictionary *)folderIDs;

-(NSArray *) foldersContainingNote:(SBNote *) note;

-(BOOL)decryptPrivateNotesWithPassword:(NSString *)oldPassword andEncryptWithNewPassword:(NSString *)newPassword;

-(BOOL) deleteAndRemoveNotes: (NSArray*) notes fromBook: (SBFolder*) book 
			progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;

-(BOOL) deleteNotes:(NSArray *)notes;	// slow method, use only if you have to
-(BOOL) deleteNotes: (NSArray*) notes inFolder: (SBFolder*) folder 
   progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;

// progressCallbackSelector needs to take one argument which is the note number being duplicated
-(NSArray *) duplicateNotes: (NSArray*) notes inBook: (SBFolder*) book 
		   progressDelegate: (id) delegate 
		   progressCallback: (SEL) progressCallbackSelector;

- (SBNote *)firstNoteCreatedAfterDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;
- (SBNote *)firstNoteCreatedBeforeDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;
- (SBNote *)firstNoteCreatedOnDay:(NSDate *)date inFolder:(SBFolder *)folder children:(BOOL)searchChildren;

-(NSData *)loadContentsForNote: (long long)noteID;
-(NSString *)loadSearchableContentsForNote: (long long)noteID;

-(BOOL) moveNotes: (NSArray*) notes fromBook: (SBFolder*) book
	 toBookWithID: (long long) bookID progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;

// Remove note from all folders it belongs too and add it the book with ID bookID
-(BOOL) moveNotes: (NSArray*) notes toBookWithID: (long long) bookID progressDelegate: (id) delegate 
 progressCallback: (SEL) progressCallbackSelector;

// Move notes to trash from all folders it belongs to
-(BOOL) moveNotesToTrash: (NSArray*) notes progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;

-(BOOL) moveNotesToTrash: (NSArray*) notes fromFolder: (SBFolder*) folder 
		progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;

-(BOOL)note: (SBNote *)note isInBook: (SBFolder *)book;
-(SBNote *)noteWithID: (long long)noteID;

// Contact may be nil to not filter by contact
-(NSArray *)notesForFolder: (SBFolder *)folder 
				forContact: (id)contact 
				  sortedBy: (NSString *)sortKey 
				descending: (BOOL)descending
			  privateNotes: (BOOL)privateNotes
		  onlyFlaggedNotes: (BOOL)onlyFlaggedNotes;

-(NSArray *)notesForFolder: (SBFolder *)folder 
				forContact: (id)contact 
				  sortedBy: (NSString *)sortKey 
				descending: (BOOL)descending 
			  privateNotes: (BOOL)privateNotes
		  onlyFlaggedNotes: (BOOL)onlyFlaggedNotes
				subFolders: (BOOL)subFolders;

/*
 Retrieve the notes assigned to a folder
 
 Parameters
 folder					The folder to which the notes are assigned
 contact				The contact the notes are assigned to, can be either an ABPerson or PGPerson contact. May be nil if no contact
 keys                   The names of the attributes to load. Nil to get all attributes
 sortedBy				The attribute to sort the notes by
 descending				Yes if the notes should be sorted in descending order
 subFolders				Yes if the notes in the child folders of folder should be read as well.
 includeCommunications	Yes if the communication log for the note should be loaded
 privateNotes			Yes if private notes should be loaded
 */
-(NSArray*) notesForFolder: (SBFolder *)folder 
				forContact: (id)contact
                      keys: (NSArray *)keys
				  sortedBy: (NSString *)sortKey 
				descending: (BOOL)descending 
			  privateNotes: (BOOL)privateNotes
          onlyFlaggedNotes: (BOOL)onlyFlaggedNotes
				subFolders: (BOOL)subFolders 
	 includeCommunications: (BOOL)includeCommunications;

-(NSArray *) duplicateNotesForFolder:(SBFolder *) folder
								keys:(NSArray *) keys
							sortedBy:(NSString *) sortKey
						  descending:(BOOL) descending
						  subFolders:(BOOL) subFolders;

-(NSArray *) notesWithIDs:(CHLongLongIndexSet *) noteIDs;
-(NSArray *)notesWithShortcuts;

-(void)startQueryNotesForSmartFolder:(SBSmartFolder *)folder 
							 contact:(id)contact
							sortedBy:(NSString *)sortKey
						  descending:(BOOL)descending
						privateNotes:(BOOL)privateNotes
					onlyFlaggedNotes:(BOOL)onlyFlaggedNotes;

/*
 Retrieve the notes assigned to a folder that contain some text
 
 Parameters
 searchText		The text to look for
 folder			The folder to which the notes are assigned
 contact		The contact the notes are assigned to, can be either an ABPerson or PGPerson contact. May be nil if no contact
 keys           The names of the notes attributes to load. Nil to get all attributes
 sortedBy		The name of the attribute to sort the notes by
 descending		Yes if the notes should be sorted in descending order
 searchOptions	Options for the search like case insensitive, etc.
 privateNotes	Yes if private notes should be loaded
 */
-(NSArray *)notesWithText: (NSString *)searchText 
				forFolder: (SBFolder *)folder 
			   forContact: (id)contact
                     keys: (NSArray *)keys
				 sortedBy: (NSString *)sortKey 
			   descending: (BOOL)descending 
			 privateNotes: (BOOL)privateNotes
		 onlyFlaggedNotes: (BOOL)onlyFlaggedNotes
			searchOptions: (SearchOptions) options;

-(BOOL) removeNotes: (NSArray*) notes fromBook: (SBFolder*) book 
   progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;
-(void)sortNotes: (NSMutableArray *)notes usingKey: (NSString *)key descending: (BOOL)descending; 
-(BOOL) trashAndRemoveNotes: (NSArray*) notes fromFolder: (SBFolder*) book 
		   progressDelegate: (id) delegate progressCallback: (SEL) progressCallbackSelector;


// Note attribute methods
//////////////////////////

// colors can be a long long as an NSNumber or an array of NSNumbers
-(BOOL) setColorOfNotes: (NSArray*) notes to: (id) colorIDs;

// flags can be an NSNumber or array of NSNumber (BOOL)
-(BOOL) setFlagsOfNotes: (NSArray*) notes to: (id) flags;

// floats can be an NSNumber or array of NSNumber (BOOL)
-(BOOL) setFloatsOfNotes: (NSArray*) notes to: (id) floats;

// labels can be a long long as a NSNumber or an array of NSNumbers
-(BOOL) setLabelsOfNotes: (NSArray*) notes to: (id) labels;

// opacities can be an NSNumber or array of NSNumber (float)
-(BOOL) setOpacitiesOfNotes: (NSArray*) notes to: (id) opacities;

// pictures and textures can be an NSString or an array of NSString
-(BOOL) setPictures: (id) pictures andTextures: (id) textures forNotes: (NSArray*) notes;

// privates can be an NSNumber or array of NSNumber (BOOL)
-(BOOL) setPrivatesOfNotes: (NSArray*) notes to: (id) privates;

// windowTypes can be an NSNumber or array of NSNumber
-(BOOL) setWindowTypesOfNotes: (NSArray*) notes to: (id) windowTypes;

-(BOOL) updateColorForNote: (SBNote*) note;
-(BOOL) updateContentsForNote: (SBNote*) note;
-(BOOL) updateFlaggedForNote: (SBNote*) note;
-(BOOL) updateFloating: (SBNote*) note;
-(BOOL) updateFrameForNote: (SBNote*) note;
-(BOOL) updateHotKeyForNote: (SBNote*) note;
-(BOOL) updateLabelForNote: (SBNote*) note;
-(BOOL) updateNote: (SBNote*) note;
-(BOOL) updateNote: (SBNote*) note updateContents: (BOOL) updateContents;
-(BOOL) updateNote: (SBNote*) note updateContents: (BOOL) updateContents updateUserAttributes: (BOOL) updateUserAttributes;
-(BOOL) updateNoteAttributesForNote:(SBNote*)note;
-(BOOL) updateMarginSettingsForNote: (SBNote*) note;
-(BOOL) updateMetadataForNote: (SBNote*) note;
-(BOOL) updateOpacity: (SBNote*) note;
-(BOOL) updatePictureAndTextureForNote: (SBNote*) note;
-(BOOL) updatePrivateForNote: (SBNote*) note;
-(BOOL) updateSelectionForNote: (SBNote*) note;
-(BOOL) updateUserAttributesForNote: (SBNote*) note;
-(BOOL) updateWindowTypeForNote: (SBNote*) note;

// OpenBase notifications:
//	This methods are called by the SBOpenBaseNotificationHandler to tell the library about
//	external changes
//
-(void)noteWasAddedExternally:(long long)noteID;
-(void)noteWasChangedExternally:(long long)noteID;
-(void)noteWasDeletedExternally:(long long)noteID;

-(void)noteWasAddedToFolderExternally:(long long)noteBooksRowID;
-(void)noteWasMovedToFolderExternally:(long long)noteBooksRowID;
-(void)noteWasRemovedFromFolderExternally:(long long)noteBooksRowID;

@end
