//
//  SBLibraryManager.h
//  StickyBrain3
//
//  Created by Rod Schmidt on Fri Feb 06 2004.
//  Copyright (c) 2004 Chronis, LC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AddressBook/ABPerson.h>
#import "CHTypes.h"

@class CHDatabase;
@class PGPerson; 
@class SBFolder;
@class SBLibrary;
@class SBNote;
@class SBOpenBaseNotificationHandler;

@interface SBLibraryManager : NSObject 
{
    NSMutableArray *                _libraries;
	SBOpenBaseNotificationHandler*	_notificationHandler;
	id								_progressDelegate;
	SEL								_progressCallback;
	BOOL							_progressResponds;
}

// General methods
//////////////////////////

+(SBLibraryManager *)sharedManager;

+(BOOL)isDatabaseSoftwareInstalled;
-(id)externalNotificationHandler;
-(void)setProgressDelegate: (id)delegate callback: (SEL)progressCallback;
-(void)updateProgress: (int) noteNumber;

// Library methods
///////////////////

-(NSEnumerator *)additionalLibrariesEnumerator;
-(BOOL)isAlreadyConnectedToLibrary: (NSDictionary *)connectionInfo;
-(NSEnumerator *)libraryEnumerator;
-(SBLibrary *)libraryForDatabase: (CHDatabase *)db;
-(SBLibrary *)libraryForDatabaseWithID: (NSString *)databaseID userID: (NSString *)userID;
-(SBLibrary *)libraryForDatabaseWithPersistentID: (NSString *)persistentID userID: (NSString *)userID;
-(SBLibrary *)libraryForFolder: (SBFolder *)folder;
-(SBLibrary *)libraryForNote: (SBNote *)aNote;
-(SBLibrary *)localLibrary;
-(int)numberOfAdditionalLibraries;
-(unsigned)numberOfLibrariesWithSameNameAsLibrary: (SBLibrary *)aLibrary;

// Database methods
////////////////////

-(CHDatabase*) databaseForFolder: (SBFolder*) aFolder;

// Privilege methods
/////////////////////

-(void) clearAccessLevels;

// Folder methods
//////////////////

-(BOOL)isAdditionalLibraryFolder: (SBFolder *)folder;
-(void)readFolders;
-(BOOL)smallIcons;
-(void)setSmallIcons: (BOOL) useSmallIcons;
-(void)updateFolderNoteCounts; 

@end

// Notifications
/////////////////

extern NSString * const SBLibraryManagerFolderWasChanged;
	// userInfo: (SBFolder*) folder - the folder that was changed

extern NSString * const SBLibraryManagerFolderWasDeleted;
extern NSString * const SBLibraryManagerDatabaseWillBeRemoved;
extern NSString * const SBLibraryManagerFoldersWereMoved;
extern NSString * const SBLibraryManagerFolderWasAdded;
    // (NSString *) folderID - ID of the added folder, nil if unknown (added remotely)

extern NSString * const SBLibraryManagerIconSizeChanged;
extern NSString * const SBLibraryManagerIconWasChanged;
extern NSString * const SBLibraryManagerLinkWasRemoved;

extern NSString * const SBLibraryManagerLinkWasRemovedFromPerson;
    // userinfo: (SBNote*) note - the note who's link was removed

extern NSString * const SBLibraryManagerLocalDatabaseDidRestore;
extern NSString * const SBLibraryManagerLocalDatabaseWillRestore;

extern NSString * const SBLibraryManagerNotesWereAdded;
	// userInfo: (NSArray*) notes - the notes that were added. This is optionally set.

extern NSString * const SBLibraryManagerNotesWereChanged;
	// userinfo: (NSArray*) noteIDs - the IDs of the notes that were changed
	//			 (NSArray*) notes - the notes that were changed
	//           userInfo will contain one or the other, but not both.
	//			 contentsChanged - boolean set to YES if the contents were changed

extern NSString * const SBLibraryManagerNotesWereDeleted;
	// userinfo:	(NSArray*) noteIDs - the IDs of the notes that were deleted

extern NSString * const SBLibraryManagerNotesWereLinked;

extern NSString * const SBLibraryManagerNotesWereLinkedToPerson;
	// userinfo: (NSArray*) noteIDs - the IDs of the notes that were changed
	//			 (NSArray*) notes - the notes that were changed
	//           userInfo will contain one or the other, but not both.
	//			 (NSArray*) keys will contain names of attributes that changed

extern NSString * const SBLibraryManagerNotesWereMoved;
extern NSString * const SBLibraryManagerNotesWereRemoved;
extern NSString * const SBLibraryManagerNotesWereRemovedExternally;

extern NSString * const SBLibraryManagerNoteWasAdded;
	// userInfo: (NSNumber) noteID - the row ID of the note that was added
	//			 (NSNumber) folderID - the row ID of the folder the note was added to.

extern NSString * const SBLibraryManagerNoteWasChanged;
	// userinfo:	(SBNote*) note - the note that was changed
	//				contentsChanged - boolean set to YES if the contents were changed

extern NSString * const SBLibraryManagerNoteWasChangedExternally;
	// userinfo:	(SBNote*) note - the note that was changed

extern NSString * const SBLibraryManagerNotesWereDeletedExternally;

extern NSString * const SBLibraryManagerPrivilegeWasChanged;
    // userinfo: (NSNumber) folderID - the row Id of the folder whose privilege was changed

extern NSString * const SBLMColorWasAddedExternallyNotification;
    // userinfo: (SBColor *) color - the color that was added/modified

extern NSString * const SBLMColorWasDeletedExternallyNotification;
    // userinfo: (NSNumber *) deletedColorID - the row ID of the color that was deleted

extern NSString * const SBLMColorWasModifiedExternallyNotification;
    
extern NSString * const SBLMLabelWasAddedExternallyNotification;
    // userinfo: (SBLabelInfo*) label - the label that was added/modified
    
extern NSString * const SBLMLabelWasDeletedExternallyNotification;
    // userinfo: (NSNumber *) deletedLabelID - the row ID of the label that was deleted

extern NSString * const SBLMLabelWasModifiedExternallyNotification;

extern NSString * const SBLMIconWasAddedExternallyNotification;
    // userinfo: (NSNumber *) rowID - the row ID of the icon that was added

extern NSString * const SBLMIconWasDeletedExternallyNotification;
    // userinfo: (NSNumber *) rowID = the row ID of the icon that was deleted

extern NSString * const SBLMIconWasModifiedExternallyNotification;
	// userinfo: (NSNumber *) rowID - the row ID of the icon that was modified

extern NSString * const SBLibraryManagerSmartFolderResultsReadyNotification;
	// userinfo: (NSArray *) notes - the notes the smart folder found

extern NSString * const SBLibraryManagerMoreSmartFolderResultsReadyNotification;
	// userinfo: (NSArray *) notes - the notes the smart folder found

extern NSString * const SBLMFormWasAddedExternallyNotification;
// userinfo: (NSNumber *) rowID - the row ID of the form that was added

extern NSString * const SBLMFormWasDeletedExternallyNotification;
	// userinfo: (NSNumber *) rowID = the row ID of the form that was deleted

extern NSString * const SBLMFormWasModifiedExternallyNotification;
	// userinfo: (NSNumber *) rowID - the row ID of the form that was modified

extern NSString * const SBLMFormWasAddedNotification;
	// userinfo: (NSNumber *) rowID - the row ID of the form that was added

extern NSString * const SBLMFormWasDeletedNotification;
	// userinfo: (NSNumber *) rowID - the row ID of the form that was deleted

extern NSString * const SBLMFormWasModifiedNotification;
	// userinfo: (NSNumber *) rowID - the row ID of the form that was modified

extern NSString * const SBLibraryManagerSearchResultsReadyNotification;
extern NSString * const SBLibraryManagerMoreSearchResultsReadyNotification;
	// userInfo: (NSArray *) notes - the notes the search found

extern NSString * const SBLMKeywordWasAddedExternallyNotification;
// userinfo: (NSNumber *) rowID - the row ID of the keyword that was added

extern NSString * const SBLMKeywordWasDeletedExternallyNotification;
// userinfo: (NSNumber *) rowID = the row ID of the keyword that was deleted

extern NSString * const SBLMKeywordWasModifiedExternallyNotification;
// userinfo: (NSNumber *) rowID - the row ID of the keyword that was modified

