//
//  SBNote.h
//  StickyBrain3
//
//  Created by Rod Schmidt on Wed Jan 21 2004.
//  Copyright (c) 2004 Chronos, LC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "CHTypes.h"
#import "CHFormDefinition.h"

@class DBRecord, OpenBase, SBLibrary;

#define MAX_SUMMARY_LENGTH  255

typedef enum {
	Inches,
	Centimeters,
	Points
} MarginUnits;

typedef enum {
	WrapToWindow,
	WrapToPage,
	StickyWindow,
	PDFWindowType,
	ImageWindowType,
	MovieWindowType,
	WebArchiveWindowType,
	AudioWindowType,
	BookmarkWindowType,
	FileWindowType,
	FormWindowType,
} SBWindowType;

typedef enum {
	TextNote,
	OutlineNote,
	FreeFormNote
} SBNoteType;

typedef enum {
	NoAction,
	GrabSelection,
	GrabSelectionNoStyle,
	PasteNote,
	PasteNoteNoStyle,
} SBHotKeyAction;

typedef enum {
	IncomingCall = 1, 
	OutgoingCall, 
	IncomingSMS, 
	OutgoingSMS	
} SBNoteCommunicationType;

extern NSString * const SBNoteCommunicationTypeKey;			// NSNumber containing an int
extern NSString * const SBNoteCommunicationStartDateKey;	// NSDate
extern NSString * const SBNoteCommunicationEndDateKey;		// NSDate
extern NSString * const SBNoteCommunicationBillingTimeKey;	// NSNumber containing a double
extern NSString * const SBNoteCommunicationPhoneNumberKey;	// NSString

extern NSString* SBHotKeyActionString(SBHotKeyAction action);

@interface SBNote : NSObject <NSCoding, NSCopying>
{
	NSString*				_uuid;
	long long				_rowid;
	long					_version;
	NSString*				_createdBy;
	NSString*				_modifiedBy;
	NSString*				_title;
	SBWindowType			_windowType;
	SBNoteType				_noteType;
	long long				_colorID;
	NSCalendarDate*			_created;
	NSTimeInterval			_modified;
	BOOL					_modifiedWasChanged;
	NSCalendarDate*			_alarm;		// alarm for current user. Not in Notes table
	BOOL					_private;
	short					_hotkey;
	long					_hotkeyModifiers;
	SBHotKeyAction			_hotkeyAction;
	BOOL					_includeInDock;
	BOOL					_shared;
	NSData*					_contents;
	NSAttributedString*		_contentsAsString;
	NSImage*				_contentsAsImage;
	BOOL					_contentsWereLoaded;
	NSString*				_bookNames;
	NSArray*				_folderIDs;
	NSMutableDictionary*	_folderAccessLevels;
	int						_linkCount;
	long					_size;
	NSAttributedString*		_summary;
	NSString*				_plainText;
	NSRange					_selection;
	double					_leftMargin;
	double					_rightMargin;
	double					_topMargin;
	double					_bottomMargin;
	NSRect					_windowFrame;
	NSRect					_expandedFrame;
	BOOL					_collapsed;
	NSString*				_pictureName;
	NSString*				_textureName;
	BOOL					_floating;
	float					_opacity;
	NSURL*					_url;
	BOOL					_dynamicTitle;
	NSDictionary*			_communication;
	NSMutableDictionary*	_metaData;
	BOOL					_flagged;
	long long				_labelID;
	long long				_formID;
	CHFormDefinition *		_formDef;
	NSMutableArray *		_keywords;
	BOOL					_encrypted;
	
	DBRecord*	_record;
	NSString*	_dbid;			// This is only used if the record is nil, which can happen after a note is encoded
	NSString*	_dbUserID;
	BOOL	_uuid_IsLoaded;//
	BOOL	_rowid_IsLoaded;//
	BOOL	_version_IsLoaded;//
	BOOL	_createdBy_IsLoaded;
	BOOL	_modifiedBy_IsLoaded;
	BOOL	_title_IsLoaded;//
	BOOL	_windowType_IsLoaded;//
	BOOL	_color_IsLoaded;//
	BOOL	_created_IsLoaded;//
	BOOL	_modified_IsLoaded;//
	BOOL	_private_IsLoaded;//
	BOOL	_hotkey_IsLoaded;//
	BOOL	_shared_IsLoaded;					// Not currently lazily loaded
	BOOL	_bookNames_IsLoaded;				// Not currently lazily loaded
	BOOL	_linkCount_IsLoaded;				// Not currently lazily loaded
	BOOL	_size_IsLoaded;//
	BOOL	_summary_IsLoaded;					// Not currently lazily loaded
	BOOL	_plainText_IsLoaded;				// Not currently lazily loaded
	BOOL	_selection_IsLoaded;//
	BOOL	_leftMargin_IsLoaded;//
	BOOL	_rightMargin_IsLoaded;//
	BOOL	_topMargin_IsLoaded;//
	BOOL	_bottomMargin_IsLoaded;//
	BOOL	_windowFrame_IsLoaded;//
	BOOL	_expandedFrame_IsLoaded;//
	BOOL	_collapsed_IsLoaded;//
	BOOL	_pictureName_IsLoaded;//
	BOOL	_textureName_IsLoaded;//
	BOOL	_floating_IsLoaded;//
	BOOL	_opacity_IsLoaded;//
	BOOL	_url_IsLoaded;//
	BOOL	_dynamicTitle_IsLoaded;
	BOOL	_isAbridged;
	BOOL	_metaDataLoaded;
	BOOL	_flaggedIsLoaded;
	BOOL	_labelIsLoaded;
	BOOL	_formIsLoaded;
}

+(NSData *)decryptContents:(NSData *)contents withPassword:(NSString *)aPassword;
+(BOOL)canDecryptContents:(NSData *)contents withPassword:(NSString *)aPassword;

// Initialization methods
//////////////////////////

-(id) initWithDBRecord:(DBRecord*)aDBRecord;

// Database methods
////////////////////

-(BOOL) isAbridged;
-(void) setAbridged: (BOOL) abridged;

-(OpenBase*) dbConnection;
-(BOOL) isNew;
-(void) setDBRecord: (DBRecord*) aDBRecord;

-(NSString*) databaseID;
-(NSString*) userID;
-(void) setDatabaseID: (NSString*) databaseUUID userID: (NSString*) userID;

-(SBLibrary *)library;
-(BOOL) load;

-(long long) noteID;
-(void) setNoteID: (long long) noteID;

-(double) timestamp;

-(NSString*) uuid;
-(void) setUUID: (NSString*) uuid;

-(long) version;
-(BOOL) hasSameVersion: (SBNote*) aNote;
-(void) setVersion: (long) version; // Only the database should use this method

// Contents methods
///////////////////

+(NSAttributedString*) stripWhitespace: (NSAttributedString*) aString truncate: (BOOL) truncate;

-(int) characterCount;

-(NSData*) contents;
-(void) setContents: (NSData*) contents;

-(NSImage*) contentsAsImage;
-(void) setContentsFromImageFile: (NSString*) filename;

-(NSAttributedString*) contentsAsString;
-(void) setContentsAsString: (NSAttributedString*) contents;

-(NSXMLDocument *)contentsAsFormXML;
-(void)setContentsAsFormXML:(NSXMLDocument *)xml;

-(NSString*) plainText;
-(void) setPlainText: (NSString*) s;

-(long) size;
-(void) setSize: (long) size;

-(NSAttributedString*) summary;
-(void) setSummary: (NSAttributedString*) summary;

-(int) wordCount;

-(BOOL)encrypt;
-(BOOL)decrypt;
-(BOOL)canDecryptWithPassword:(NSString *)password;

// Emailing Notes
//////////////////

-(NSString*) emailBody;
-(NSString*) emailBodyForKnownClients;

// File methods - for writing a note's contents to a file
//////////////////////////////////////////////////////////

-(NSString*) filename;
-(NSString*) pathExtension;

// Folder methods
/////////////////

-(void) setAccessLevel: (CHAccessLevel) accessLevel forFolder: (long long) folderID;

-(NSString*) bookNames;
-(NSArray*) arrayOfBookNames;
-(void) setBookNames: (NSString*) names;

-(NSArray*) folderIDs;
-(void) setFolderIDs: (NSArray*) folderIDs;

-(BOOL) inOnlyOneBook;
-(BOOL) inTrash;

// Linking notes
/////////////////

-(int) linkCount;
-(void) setLinkCount: (int) count;

-(void) decrementLinkCount;
-(void) incrementLinkCount;

// Object methods - for copying and comparing notes
////////////////////////////////////////////////////

-(NSComparisonResult) compare: (SBNote*) otherNote properties: (NSArray*) properties;

// Take the values in note and make them our own.
-(void) takeValuesFrom: (SBNote*) note;

// Just take certain values from note and make them our own. If keys is nil or empty will get all values
-(void) takeValuesFrom: (SBNote*) note keys: (NSArray*) keys;

// Take all the values except the contents if they haven't changed.
-(void) takeValuesFrom: (SBNote*) note contentsChanged: (BOOL) contentsChanged;

// PDF Setting methods
//////////////////////

// Mode is really PDFDisplayMode. But we don't want to import the header to avoid linking with the Quartz
// framework.
-(int) displayMode;
-(void) setDisplayMode: (int) mode;

-(NSString*) metaDataString;
-(void) setMetaDataFromString: (NSString*) settingsString;

-(unsigned int) page;
-(void) setPage: (unsigned int) page;

-(float) scale;
-(void) setScale: (float) scale;

// Bookmark Settings
/////////////////////

-(float) textSizeMultiplier;
-(void) setTextSizeMultiplier:(float) multiplier;

// Title methods
/////////////////

-(BOOL) dynamicTitle;
-(void) setDynamicTitle: (BOOL) dynamicTitle;

// Generate the note's title from its contents
-(void) generateTitle;

-(NSString*) title;
-(BOOL) titleIsEmpty;
-(void) setTitle: (NSString*) aTitle;

// Window attributes
/////////////////////

+(MarginUnits) marginUnits;

-(double) bottomMargin;
-(void) setBottomMargin: (double) bottomMargin;

-(BOOL) isCollapsed;
-(void) setCollapsed: (BOOL) collapsed;

-(long long) colorID;
-(void) setColorID: (long long) colorID;

-(NSRect) expandedFrame;
-(void) setExpandedFrame: (NSRect) frame;

-(BOOL) isFloating;
-(void) setFloating: (BOOL) floating;

-(double) leftMargin;
-(void) setLeftMargin: (double) leftMargin;

-(float) opacity;
-(void) setOpacity: (float) opacity;

-(NSString*) picture;
-(void) setPicture: (NSString*) pictureName;

-(double) rightMargin;
-(void) setRightMargin: (double) rightMargin;

-(NSString*) texture;
-(void) setTexture: (NSString*) textureName;

-(double) topMargin;
-(void) setTopMargin: (double) topMargin;

-(NSRect) windowFrame;
-(void) setWindowFrame: (NSRect) frame;

-(int) windowHeight;

-(SBWindowType) windowType;
-(BOOL) isTextType;
+(BOOL) isTextType:(SBWindowType)aWindowType;
-(void) setWindowType: (SBWindowType) aWindowType;

-(int) windowWidth;

// Forms
/////////

-(CHFormDefinition *)formDefinition;
-(void)setFormDefinition:(CHFormDefinition *)formDef;

-(long long)formID;
-(void)setFormID:(long long)formID;

// Keyword methods
/////////////////////

-(NSArray *)keywords;
-(void)setKeywords:(NSArray *)keywords;
-(NSString *)keywordsAsString;
-(void)setKeywordsAsString:(NSString *)keywordsString;
-(void)addKeyword:(NSString *)keyword;

// General accessors
/////////////////////

-(NSCalendarDate*) alarm;
-(void) setAlarm: (NSCalendarDate*) date;

-(NSDictionary*) communication;
-(void) setCommunication: (NSDictionary*) comm;

-(NSString*) createdBy;
-(void) setCreatedBy: (NSString*) userID;

-(NSCalendarDate*) creationDate;
-(void) setCreationDate: (NSCalendarDate*) date;

-(BOOL) isFlagged;
-(void) setFlagged: (BOOL) isFlagged;

-(short) hotKey;
-(long) hotKeyModifiers;
-(SBHotKeyAction) hotKeyAction;
-(BOOL) includeInDock;
-(void) setHotKey: (short) hotkey modifiers: (long) modifiers action: (SBHotKeyAction) action include:(BOOL) includeInDock;

-(long long) labelID;
-(void) setLabelID: (long long) labelID;

-(NSCalendarDate*) modificationDate;
-(void) setModificationDate: (double) d;

-(NSString*) modifiedBy;
-(void) setModifiedBy: (NSString*) userID;

-(SBNoteType) noteType;

-(BOOL) isPrivate;
-(void) setPrivate: (BOOL) isPrivate;

-(NSRange) selection;
-(void) setSelection: (NSRange) selectionRange;

-(BOOL) isShared;
-(void) setShared: (BOOL) isShared;

-(NSURL*) url;
-(void) setURL: (NSURL*) aURL;

@end
