//
//  SBSmartFolderRule.h
//  CHDatabaseKit
//
//  Created by Rod Schmidt on 10/26/06.
//  Copyright 2006 Chronos, LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef enum 
{	
	SBNoRuleAttribute,
	SBEntireNote,
	SBTitle,
	SBContents,
	SBDateCreated,
	SBDateModified,
	SBNoteLabelIs,
	SBNoteTypeIs,
	SBNoteTypeIsNot,
	SBNoteIsFlagged,
	SBNoteIsPrivate,
	SBNoteHasShortcut,
	SBNoteIsInFolder,
	SBNoteIsNotInFolder,
	SBKeywords,
} SBRuleAttribute;

typedef enum 
{
	SBNullStringOperator = 0,
	SBContains,
	SBDoesNotContain,
	SBBeginsWith,
	SBEndsWith,
	SBIsEqualTo
} SBStringOperator;

typedef enum 
{
	SBNullDateOperator = 50,	// give SBStringOperator some room to grow
	SBIsToday,
	SBIsYesterday,
	SBIsThisWeek,
	SBIsLastWeek,
	SBIsExactly,
	SBIsInTheLast,
	SBIsNotInTheLast,
	SBIsBetween,
	SBIsTheDate,
	SBIsAfterTheDate,
	SBIsBeforeTheDate,
	SBIsInTheDateRange
} SBDateOperator;

typedef enum
{
	SBDays,
	SBWeeks,
	SBMonths,
	SBYears
} SBDateUnits;

typedef enum 
{
	SBAllConditions = 1,
	SBAnyCondition
} SBSearchLogic;

@interface SBSmartFolderRule : NSObject <NSCoding>
{
	long long _rowID;
	long long _folderID;
	NSString *_uuid;
	SBSearchLogic _logic;
	SBRuleAttribute _attribute;
	unsigned int _operator;
	id _value1;
	id _value2;
	SBDateUnits _dateUnits;
	
	NSMutableArray *_predicateArgs;
}

// Initialization
//////////////////

+(SBSmartFolderRule *)rule;

// A dictionary containing the columns from the rules database record
+(SBSmartFolderRule *)ruleFromDictionary:(NSDictionary *)dict;

-(id)initWithDictionary:(NSDictionary *)dict;

// Accessors
/////////////

-(long long)rowID;
-(void)setRowID:(long long)rowID;

-(long long)folderID;
-(void)setFolderID:(long long)aFolderID;

-(NSString *)uuid;
-(void)setUUID:(NSString *)aUUID;

-(SBSearchLogic)logic;
-(void)setLogic:(SBSearchLogic)logic;

-(SBRuleAttribute)attribute;
-(void)setAttribute:(SBRuleAttribute)anAttribute;

-(unsigned int)operator;
-(void)setOperator:(unsigned int)anOperator;

-(id)value1;
-(NSString *)value1AsString;
-(void)setValue1:(id)aValue;

-(id)value2;
-(NSString *)value2AsString;
-(void)setValue2:(id)aValue;

-(SBDateUnits)dateUnits;
-(void)setDateUnits:(SBDateUnits)units;

// Other methods
/////////////////

+(BOOL)isDateAttribute:(SBRuleAttribute)attribute;
+(BOOL)isFolderAttribute:(SBRuleAttribute)attribute;
+(BOOL)isLabelAttribute:(SBRuleAttribute)attribute;
+(BOOL)isNoteTypeAttribute:(SBRuleAttribute)attribute;
+(BOOL)isStringAttribute:(SBRuleAttribute)attribute;

-(NSPredicate *)predicate;

@end
