/*
 *  CHObjCRuntime.h
 *  CHFoundation
 *
 *  Created by Nick Zitzmann on 10/3/05.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#include <Foundation/Foundation.h>

// Sometimes +initialize methods are called twice. This tends to happen if a subclass of a class implements +initialize but the class doesn't. The following macro fixes this problem. Put this in as the first thing in +initialize methods:
// CH_INITIALIZE_ONCE;
// ... to activate it.
#define CH_INITIALIZE_ONCE	static BOOL alreadyInitialized = NO; if (alreadyInitialized) return; alreadyInitialized = YES

// NSInteger is a type that was added to Leopard.
// Here is some glue to ensure that NSInteger will work with pre-10.5 SDKs:
#ifndef NSINTEGER_DEFINED
#ifdef NS_BUILD_32_LIKE_64
typedef long NSInteger;
typedef unsigned long NSUInteger;
#else
typedef int NSInteger;
typedef unsigned int NSUInteger;
#endif
#define NSIntegerMax    LONG_MAX
#define NSIntegerMin    LONG_MIN
#define NSUIntegerMax   ULONG_MAX
#define NSINTEGER_DEFINED 1
#endif

// CGFloat is a type that was added to Leopard.
// Here is some glue to ensure that CGFloat will work with pre-10.5 SDKs:
#ifndef CGFLOAT_DEFINED
typedef float CGFloat;
#define CGFLOAT_MIN FLT_MIN
#define CGFLOAT_MAX FLT_MAX
#define CGFLOAT_IS_DOUBLE 0
#define CGFLOAT_DEFINED 1
#endif

BOOL CHIsImageWithPathLoaded(const char *imagePath);
void CHLogf(int descriptor, NSString *format, ...);
void CHLogfv(int descriptor, NSString *format, va_list args);
