//
//  NSArray+CHExtensions.h
//  CHFoundation
//
//  Created by Nick Zitzmann on 03/30/2005.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#ifndef NSINTEGER_DEFINED
#import "CHObjCRuntime.h"
#endif

@interface NSArray (CHExtensions)
- (BOOL)allHaveSameKey:(NSString *)key;
- (BOOL)areAllObjectsEqual:(id)equalObject;
- (NSArray *)arrayByInsertingObject:(id)object atIndex:(unsigned int)index;
- (NSArray *)arrayByRemovingObject:(id)anObject;
- (NSArray *)arrayByRemovingObjectsFromArray:(NSArray *)otherArray;
- (NSArray *)arrayOfObjectsInDictionariesForKey:(id)aKey;   // in an array of dictionaries, returns an array containing the objects in each dictionary with key "aKey"
- (char * const *)cArrayUsingEncoding:(NSStringEncoding)encoding;	// return value is autoreleased; array must be composed entirely of NSStrings
- (BOOL)containsAtLeastOneDictionaryWithObjectForKey:(id)aKey;
- (id)deepMutableCopy;
- (id)deepMutableCopyWithZone:(NSZone *)aZone;

// Selector should take one argument which is of the type the array contains
- (void)doForEach: (SEL) selector target: (id) target;

- (NSUInteger)fastFindInsertIndexForObject:(id)anObject properties:(NSArray *)properties;	// WARNING: objects must comply with the PGFastSearching protocol, see the end of this header file
- (NSUInteger)fastFindInsertIndexForObject:(id)anObject comparisonSelector:(SEL)selector;
- (NSUInteger)fastFindInsertIndexForObject:(id)anObject cComparisonFunction:(int (*)(id, id, void *))comparator;
- (NSUInteger)fastFindInsertIndexForObject:(id)anObject cComparisonFunction:(int (*)(id, id, void *))comparator context:(void *)context;
- (NSUInteger)fastIndexOfObject:(id)anObject properties:(NSArray *)properties;	// WARNING: objects must comply with the PGFastSearching protocol, see the end of this header file
- (NSUInteger)fastIndexOfObject:(id)anObject comparisonSelector:(SEL)selector;
- (NSUInteger)fastIndexOfObject:(id)anObject cComparisonFunction:(int (*)(id, id, void *))comparator;
- (NSUInteger)fastIndexOfObject:(id)anObject cComparisonFunction:(int (*)(id, id, void *))comparator context:(void *)context;
- (id)firstObject;
- (NSUInteger)indexOfDictionaryWithObject:(id)anObject forKey:(id)aKey;   // returns NSNotFound if there are no dictionaries in the array with object "anObject" for key "aKey"
- (id)initWithObject:(id)anObject;
- (BOOL)intersectsArray:(NSArray *)otherArray;
- (BOOL)isMutable;
- (void)makeObjectsPerformSelector:(SEL)selector withBool:(BOOL)boolean;
- (void)makeObjectsPerformSelector:(SEL)selector withInt:(int)integer;
- (void)makeObjectsPerformSelector:(SEL)selector withLongLong:(long long)longLongInteger;
- (void)makeObjectsPerformSelector:(SEL)selector withObject:(id)object1 withObject:(id)object2;
- (void)makeObjectsPerformSelector:(SEL)selector withSelector:(SEL)selectorArg;
- (id)searchForDictionaryWithObject:(id)anObject forKey:(id)aKey;
@end

@interface NSMutableArray (CHExtensions)
- (void)addObjects:(id)firstArg, ...;   // takes a nil-terminated list of objects, like NSArray's +arrayWithObjects: method
- (void)mergeObjectsFromArray:(NSArray *)array;
- (void)moveObject:(id)object toIndex:(NSUInteger)destination;
- (void)moveObjectAtIndex:(NSUInteger)source toIndex:(NSUInteger)destination;
- (void)removeObjectsInArray:(NSArray *)array usingComparisonSelector:(SEL)comparisonSel;	// comparisonSel must point to a method that returns an NSComparisonResult
- (void)replaceObject:(id)anObject withObject:(id)anotherObject;
@end
