//
//  CHDatabaseManagerController.h
//  SOHO Notes
//
//  Created by Rod Schmidt on 3/11/2005.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CHAppKit/CHAppKit.h>

@class CHDatabase;

#define CHDatabaseManagerDatabasesSavedNotification	@"CHDatabaseManagerDatabasesSavedNotification"
#define CalendarDatabasesToSyncChangedNotification	@"CalendarDatabasesToSyncChangedNotification"

// The userinfo for these notifications contains the connection info for the affected database
// name, host, and login
#define CHDatabaseWillGoOfflineNotification	@"CHDatabaseWillGoOfflineNotification"
#define CHDatabaseDidGoOfflineNotification	@"CHDatabaseDidGoOfflineNotification"
#define CHDatabaseWillGoOnlineNotification	@"CHDatabaseWillGoOnlineNotification"
#define CHDatabaseDidGoOnlineNotification	@"CHDatabaseDidGoOnlineNotification"

extern NSString * const CHDatabaseManagerDatabaseMustRemoveNotification;
extern NSString * const CHDatabaseManagerDatabaseCanReconnectNotification;

// rob - @interface CHDatabaseManagerController : NSWindowController <SubviewTableViewControllerDataSourceProtocol>
@interface CHDatabaseManagerController : NSObject <SubviewTableViewControllerDataSourceProtocol>
{
	IBOutlet NSTableView*	_table;
	IBOutlet NSButton*		_addButton;
	IBOutlet NSButton*		_removeButton;
	IBOutlet NSButton*		_onlineOfflineButton;
	IBOutlet NSButton*		_changePasswordButton;
	IBOutlet NSView*		_mainView;		// rob
	
	IBOutlet NSWindow*		_changePasswordDialog;
	IBOutlet NSWindow*		_newPasswordDialog;
	IBOutlet NSSecureTextField*	_oldPasswordField;
	IBOutlet NSSecureTextField*	_newPasswordField;
	IBOutlet NSSecureTextField*	_confirmPasswordField;
	IBOutlet NSButton*		_oldPasswordOKButton;
	IBOutlet NSButton*		_newPasswordOKButton;
	
	NSMutableArray* _topLevelNibObjects;	// rob
	NSMutableArray*	_databases;
	NSRecursiveLock *_databasesLock;
	SubviewTableViewController *_cellController;
	id _delegateForDatabases;
}

+(CHDatabaseManagerController*) sharedDatabaseManager;

- (id)mainView;		// rob

-(IBAction) addDatabase: sender;
-(IBAction) removeDatabaseAction: sender;
- (void)removeDatabase:(CHDatabase*)db;
-(IBAction) toggleOnline: sender;

- (IBAction)changePassword:(id)sender;
- (IBAction)oldPasswordButtonAction:(id)sender;
- (IBAction)newPasswordButtonAction:(id)sender;

-(void) restoreConnectedDatabases;
-(void) saveConnectedDatabases;

- (void)setDelegateForDatabases:(id)newDelegateForDatabases;

@end

@protocol CHDatabaseManagerControllerDelegate
//-(CHDatabase*) database;
-(void) connectToDatabase: (id) sender;
-(BOOL) connectToDatabase: (NSString*) dbName host: (NSString*) host login: (NSString*) login 
			  directLogin: (BOOL) directLogin;
-(void) newDatabase: sender;
@end
