//
//  PGBlockHolderView.h
//  CHOpenBaseGUI
//
//  Created by Jerry Halls on Wed Dec 17 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <AppKit/AppKit.h>
#import <CHAppKit/CHAppKit.h>
#import <CHDatabaseKit/CHDatabaseKit.h>

@class PGBlockView, PGBlockContentViewController, PGTheme;

@interface PGBlockHolderView : NSView
{
	int				_minNumColumns;
	int				_maxNumColumns;
	float			_minColumnWidth;
	float			_maxColumnWidth;
	NSMutableArray  *_blockViewsArray;
	PGTheme			*_theme;
	
	BOOL _isPageHolder;
	BOOL _blockViewIsCentered;
	id _record;
	NSArray *_personAddedProperties;
	NSArray *_personOverwrittenProperties;
	
	float _zoomFactor;
	CHCornerBannerView *_bannerView;
}

- (id)initWithFrame:(NSRect)frame isPageHolder:(BOOL)isHolder;

- (void)addBlockView:(PGBlockView*)aBlockView;
- (PGBlockView *)addBlockViewWithFrame:(NSRect)theRect title:(NSString *)title contentViewController:(PGBlockContentViewController *)controller expanded:(BOOL)expanded columnsSpanned:(unsigned int)columnsSpanned blockID:(unsigned int)blockID;
- (NSArray *)blockViewArray;
- (PGBlockView *)blockViewAtIndex:(unsigned int)index;
- (NSArray *)blockViewsWithContentViewControllerClass:(Class)aClass;
- (NSArray *)blockViewsWithKindOfContentViewControllerClass:(Class)aClass;
- (NSArray *)blockViewsWithBlockID:(unsigned int)blockID;
- (void)moveBlockView:(PGBlockView *)blockView toIndex:(unsigned int)index;
- (void)removeBlockViewAtIndex:(unsigned int)index;
- (void)removeAllBlockViews;
- (void)repositionAllBlockViews;
- (void)centerBlockView;

- (void)setUsesBanner:(BOOL)useBanner;
- (void)setBannerColor:(NSColor *)color;
- (void)setBannerString:(NSString *)string;

- (void)setupBlockViewsKeyViews:(PGBlockView *)aBlockView;	// this calls both -setupBlockViewsKeyViewsSearchingBackwards: and -setupBlockViewsKeyViewsSearchingForwards: and is often the safest key-view setup method to call
- (void)setupBlockViewsKeyViewsSearchingBackwards:(PGBlockView *)aBlockView;	// this connects a preceding visible block view's final key view to the initial key view of aBlockView
- (void)setupBlockViewsKeyViewsSearchingForwards:(PGBlockView *)aBlockView;	// this connects the final key view of aBlockView to the initial key view of the next visible block view
- (void)setupOutsideViewsIncomingKeyView:(NSView *)outsideView;
- (void)setupOutsideViewsOutgoingKeyView:(NSView *)outsideView;
- (void)setKeyViewsToLoopInsideBlockHolderView;

- (id)record;
- (void)setRecord:(id)record;

- (void)setPersonAddedProperties:(NSArray *)addedProperties overwrittenProperties:(NSArray *)overwrittenProperties;
- (void)loadRecordIntoViews;
- (void)setAddressBook:(PGAddressBook *)addressBook;

- (unsigned int)numberOfBlockViews;
- (unsigned int)numberOfVisibleBlockViews;
- (void)setMinNumColumns:(int)aNumColumns;
- (void)setMaxNumColumns:(int)aNumColumns;
- (void)setTheme:(PGTheme *)aTheme;
- (void)setZoomFactor:(float)zoomFactor;
- (NSMutableArray*)calculateRowHeights:(float*)aTotalHeight;

- (void)setMinColumnWidth:(float)aWidth;
- (float)minColumnWidth;

@end